/*
 *  Copyright (C) 2003-2004 Curt Coder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <string>
#include <cmath>
#include <SDL.h>
#include "agipic.h"
#include "gfx.h"

static AGI_PICTURE pic;

// debug features
static bool fFill = true;
static bool fStep = false;

#define QMAX 4000
#define EMPTY 0xFF

static Uint8 buf[QMAX+1] = {0};
static Uint16 rpos = QMAX, spos = 0;

static void qstore(Uint8 q)
{
	if (spos + 1 == rpos || (spos + 1 == QMAX && !rpos)) return;

	buf[spos] = q;
	spos++;
	
	if (spos == QMAX) spos = 0;
}

static Uint8 qretrieve()
{
	if (rpos == QMAX) rpos = 0;
	if (rpos == spos) return EMPTY;

	rpos++;

	return buf[rpos - 1];
}

/**************************************************************************
** AGI_SetPixelPic
**
** Draws a pixel in the picture screen.
**************************************************************************/
static void AGI_SetPixelPic(int x, int y)
{
	if (x >= pic.w) return;
	if (y >= IDI_AGI_PIC_MAX_HEIGHT) return;
	if (y >= pic.h) return;
	pic.bmpPic[x + pic.x0][y + pic.y0] = pic.colPic;
	PutPixelAGI(x + pic.x0, y + pic.y0, pic.colPic);
}

/**************************************************************************
** AGI_SetPixelPri
**
** Draws a pixel in the priority screen.
**************************************************************************/
static void AGI_SetPixelPri(int x, int y)
{
	if (x >= pic.w) return;
	if (y >= pic.h) return;
	pic.bmpPri[x + pic.x0][y + pic.y0] = pic.colPri;
}

/**************************************************************************
** AGI_Plot
**
** Draws a pixel in each screen depending on whether drawing in that
** screen is enabled or not.
**************************************************************************/
static void AGI_Plot(int x, int y)
{
	if (pic.fDrawPic) AGI_SetPixelPic(x, y);
	if (pic.fDrawPri) AGI_SetPixelPri(x, y);
}

/**************************************************************************
** AGI_GetPixelPic
**
** Get Color at x,y on the picture page.
**************************************************************************/
static Uint8 AGI_GetPixelPic(int x, int y)
{
	if (x >= pic.w) return(4);
	if (y >= pic.h) return(4);
	return pic.bmpPic[x + pic.x0][y + pic.y0];
}

/**************************************************************************
** AGI_GetPixelPri
**
** Get Color at x,y on the priority page.
**************************************************************************/
static Uint8 AGI_GetPixelPri(int x, int y)
{
	if (x >= pic.w) return(4);
	if (y >= pic.h) return(4);
	return pic.bmpPri[x + pic.x0][y + pic.y0];
}

/**************************************************************************
** AGI_DrawLine
**
** Draws an AGI line.
**************************************************************************/
static void AGI_DrawLine(int x0, int y0, int x1, int y1)
{
	int x = x0, y = y0;
	int dy = 1, dx = 1;
	int height, width;
	int xcount, ycount;
	int counter, maxdim;
/*
	// draw y line if y0 = y1
	if (y0 == y1) {
		if (x0 < x1)
			for (x = x0; x <= x1; x++) AGI_Plot(x, y);
		else
			for (x = x0; x <= x1; x--) AGI_Plot(x, y);
		return;
	}

	// draw x line if x0 = x1
	if (x0 == x1) {
		if (y0 < y1)
			for (y = y0; y <= y1; y++) AGI_Plot(x, y);
		else
			for (y = y0; y <= y1; y--) AGI_Plot(x, y);
		return;
	}
*/
	height = abs(y1 - y0);
	width = abs(x1 - x0);

	if (y0 > y1)
		dy = -1;
	if (x0 > x1)
		dx = -1;
	
	xcount = height / 2;
	ycount = width / 2;

	if (width > height) {
		counter = maxdim = width;
		xcount = 0;
	} else {
		counter = maxdim = height;
		ycount = 0;
	}

	for (int i = 0; i < counter; i++) {
		ycount += height;
		if (ycount >= maxdim) {
			ycount -= maxdim;
			y += dy;
		}
		xcount += width;
		if (xcount >= maxdim) {
			xcount -= maxdim;
			x += dx;
		}
		AGI_Plot(x, y);
	}
}

/**************************************************************************
** AGI_okToFill
**************************************************************************/
static bool AGI_okToFill(int x, int y)
{
	if (!pic.fDrawPic && !pic.fDrawPri) return false;
	if (pic.colPic == 15) return false;
	if (!pic.fDrawPri) return (AGI_GetPixelPic(x, y) == 15);
	if (pic.fDrawPri && !pic.fDrawPic) return (AGI_GetPixelPri(x, y) == 4);
	return (AGI_GetPixelPic(x, y) == 15);
}

/**************************************************************************
** AGI_agiFill
**************************************************************************/
/*
static void AGI_sagiFill(int x0, int y0)
{
	int x = x0, y = y0;
	Uint8 left, right, dir, toggle;
	Uint8 old_left, old_right, old_dir, old_toggle, old_initx, old_inity;
	int color0;

	if (!AGI_okToFill(x0, y0)) return;

	color0 = AGI_GetPixelPic(x0, y0);

	left = 161;
	right = 0;
	dir = 1;
	toggle = 0;

	for (;;) {
		old_right = right;
		old_left = left;
		old_toggle = toggle;
		old_initx = x0;

		// get left boundary
		x = x0;
		do {
			AGI_Plot(x, y);
			x--;
		} while ((x >= 0) && (AGI_GetPixelPic(x, y) == color0));
		left = x;

		// get right boundary
		x = x0;
		do {
			AGI_Plot(x, y);
			x++;
		} while ((x < 160) && (AGI_GetPixelPic(x, y) == color0));
		right = x;

		if (old_left != 161) {
			if (right == old_right)	{
				if (left != old_left) {
					toggle = 0;
					old_initx = old_right;
				}
				if (toggle == 1) goto locnext;
				toggle = 1;
				old_initx = right;
			} else if (right > old_right) {
				toggle = 0;
				old_initx = old_right;
			} else {
				toggle = 0;
				old_initx = right;	
			}

			qstore(old_toggle);
			qstore(dir);
			//qstore(old_direction);
			qstore(old_inity);
			qstore(old_initx);
			qstore(old_right);
			qstore(old_left);
		}
locnext:
		
	}
}
*/
static void AGI_agiFill(int x, int y)
{
	Uint8 x1, y1;
	rpos = spos = 0;

	qstore(x);
	qstore(y);

	for (;;) {

		x1 = qretrieve();
		y1 = qretrieve();

		if ((x1 == EMPTY) || (y1 == EMPTY))
			break;
		else {
			if (AGI_okToFill(x1, y1)) {
				AGI_Plot(x1, y1);
				if (AGI_okToFill(x1, y1 - 1) && (y1 != 0)) {
					qstore(x1);
					qstore(y1 - 1);
				}
				if (AGI_okToFill(x1 - 1, y1) && (x1 != 0)) {
					qstore(x1 - 1);
					qstore(y1);
				}
				if (AGI_okToFill(x1 + 1, y1) && (x1 != 159)) {
					qstore(x1 + 1);
					qstore(y1);
				}
				if (AGI_okToFill(x1, y1 + 1) && (y1 != 167)) {
					qstore(x1);
					qstore(y1 + 1);
				}
			}
		}
	}
}

/**************************************************************************
** AGI_DrawXcorner
**
** Draws an AGI_DrawXcorner  (drawing action 0xF5)
**************************************************************************/
static void AGI_DrawXcorner(Uint8 **data)
{
	Uint8 x1, x2, y1, y2;

	x1 = *((*data)++);
	y1 = *((*data)++);

	AGI_Plot(x1, y1);

	for (;;) {
		x2 = *((*data)++);
		
		if (x2 >= IDI_AGI_MIN) break;
		
		AGI_DrawLine(x1, y1, x2, y1);

		x1 = x2;
		y2 = *((*data)++);

		if (y2 >= IDI_AGI_MIN) break;

		AGI_DrawLine(x1, y1, x1, y2);

		y1 = y2;
	}

	(*data)--;
}

/**************************************************************************
** AGI_DrawYcorner
**
** Draws an AGI_DrawYcorner  (drawing action 0xF4)
**************************************************************************/
static void AGI_DrawYcorner(Uint8 **data)
{
	Uint8 x1, x2, y1, y2;

	x1 = *((*data)++);
	y1 = *((*data)++);

	AGI_Plot(x1, y1);

	for (;;) {
		y2 = *((*data)++);

		if (y2 >= IDI_AGI_MIN) break;
		
		AGI_DrawLine(x1, y1, x1, y2);
		y1 = y2;
		x2 = *((*data)++);
		
		if (x2 >= IDI_AGI_MIN) break;
		
		AGI_DrawLine(x1, y1, x2, y1);
		
		x1 = x2;
	}

	(*data)--;
}

/**************************************************************************
** AGI_DrawLineRel
**
** Draws short lines relative to last position.  (drawing action 0xF7)
**************************************************************************/
static void AGI_DrawLineRel(Uint8 **data)
{
	Uint8 x0, y0, x1, y1, disp;
	Uint8 dx, dy;

	x0 = *((*data)++);
	y0 = *((*data)++);

	AGI_Plot(x0, y0);

	for (;;) {
		disp = *((*data)++);

		if (disp >= IDI_AGI_MIN) break;
		
		dx = (disp & 0x70) / 16;
		if ((disp & 0x80) == 0)
			x1 = x0 + dx;
		else
			x1 = x0 - dx;

		dy = (disp & 0x07);
		if ((disp & 0x08) == 0)
			y1 = y0 + dy;
		else
			y1 = y0 - dy;

		AGI_DrawLine(x0, y0, x1, y1);
		
		x0 = x1;
		y0 = y1;
	}

	(*data)--;
}

/**************************************************************************
** AGI_Fill
**
** Agi flood fill.  (drawing action 0xF8)
**************************************************************************/
static void AGI_Fill(Uint8 **data)
{
	Uint8 x1, y1;

	for (;;) {
		if ((x1 = *((*data)++)) >= IDI_AGI_MIN) break;
		if ((y1 = *((*data)++)) >= IDI_AGI_MIN) break;

		if (fFill)
			AGI_agiFill(x1, y1);
		else {
			AGI_Plot(x1, y1 - 2);
			AGI_Plot(x1, y1 - 1);
			AGI_Plot(x1 - 1, y1);
			AGI_Plot(x1, y1);
			AGI_Plot(x1 + 1, y1);
			AGI_Plot(x1, y1 + 1);
			AGI_Plot(x1, y1 + 2);
		}
	}

	(*data)--;
}

/**************************************************************************
** AGI_DrawLineAbs
**
** Draws long lines to actual locations (cf. relative) (drawing action 0xF6)
**************************************************************************/
static void AGI_DrawLineAbs(Uint8 **data)
{
	Uint8 x1, y1, x2, y2;

	x1 = *((*data)++);
	y1 = *((*data)++);

	AGI_Plot(x1, y1);

	for (;;) {
		if ((x2 = *((*data)++)) >= IDI_AGI_MIN) break;
		if ((y2 = *((*data)++)) >= IDI_AGI_MIN) break;

		AGI_DrawLine(x1, y1, x2, y2);

		x1 = x2;
		y1 = y2;
	}

	(*data)--;
}

#define AGI_PlotPatternPoint() \
	if (pic.patCode & 0x20) { \
		if ((splatterMap[bitPos >> 3] >> (7 - (bitPos & 7))) & 1) AGI_Plot(x1, y1); \
		bitPos++; \
		if (bitPos == 0xff) bitPos=0; \
	} else AGI_Plot(x1, y1)

/**************************************************************************
** AGI_PlotPattern
**
** Draws pixels, circles, squares, or splatter brush patterns depending
** on the pattern code.
**************************************************************************/

static void AGI_PlotPattern(int x, int y)
{
	static Uint16 binary_list[] = {0x8000, 0x4000, 0x2000, 0x1000, 0x800, 0x400, 0x200, 0x100, 
		0x80, 0x40, 0x20, 0x10, 0x8, 0x4, 0x2, 0x1};

	static Uint8 circle_list[] = {0, 1, 4, 9, 16, 25, 37, 50};

	static Uint16 circle_data[] =
		{0x8000, 
		0xE000, 0xE000, 0xE000, 
		0x7000, 0xF800, 0x0F800, 0x0F800, 0x7000, 
		0x3800, 0x7C00, 0x0FE00, 0x0FE00, 0x0FE00, 0x7C00, 0x3800, 
		0x1C00, 0x7F00, 0x0FF80, 0x0FF80, 0x0FF80, 0x0FF80, 0x0FF80, 0x7F00, 0x1C00,
		0x0E00, 0x3F80, 0x7FC0, 0x7FC0, 0x0FFE0, 0x0FFE0, 0x0FFE0, 0x7FC0, 0x7FC0, 0x3F80, 0x1F00, 0x0E00,
		0x0F80, 0x3FE0, 0x7FF0, 0x7FF0, 0x0FFF8, 0x0FFF8, 0x0FFF8, 0x0FFF8, 0x0FFF8, 0x7FF0, 0x7FF0, 0x3FE0, 0x0F80,
		0x07C0, 0x1FF0, 0x3FF8, 0x7FFC, 0x7FFC, 0x0FFFE, 0x0FFFE, 0x0FFFE, 0x0FFFE, 0x0FFFE, 0x7FFC, 0x7FFC, 0x3FF8, 0x1FF0, 0x07C0};

	Uint16 circle_word;
	Uint16 *circle_ptr;	// si
	Uint16 counter;	// bx sometimes
	Uint16 pen_width = 0;
	Uint16 pen_final_x = 0;	// 15EA
	Uint16 pen_final_y = 0;	// 15EC
	
	Uint8 t = 0;
	Uint8 temp8;
	Uint16 temp16;

	Uint16	pen_status = pic.patCode;
	Uint16	pen_x = x;
	Uint16	pen_y = y;
	Uint16	texture_num = 0;
	Uint16	pen_size = (pen_status & 0x07);

	circle_ptr = &circle_data[ circle_list[pen_size] ];	// pen size
	
	// setup the X position
	// = pen_x - pen.size/2

	pen_x = (pen_x * 2) - pen_size;
	if (pen_x < 0) pen_x = 0;

	temp16 = 320 - (2 * pen_size);
	if (pen_x >= temp16)
		pen_x = temp16;
		
	pen_x /= 2;
	pen_final_x = pen_x;	// original starting point?? -> used in plotrelated

	// Setup the Y Position
	// = pen_y - pen.size
	pen_y = pen_y - pen_size;
	if (pen_y < 0) pen_y = 0;

	temp16 = 167 - (2 * pen_size);
	if (pen_y >= temp16)
		pen_y = temp16;
		
	pen_final_y = pen_y;	// used in plotrelated

	t = texture_num | 0x01;		// even
	
	// new purpose for temp16
	
	temp16 =( pen_size<<1) +1;	// pen size
	pen_final_y += temp16;					// the last row of this shape
	temp16 = temp16 << 1;
	pen_width = temp16;					// width of shape?

	//printf("status = %X   pen_width = %d  pen_final_y = %d  pen_x = %d   pen_y = %d\n", pen_status, pen_width, pen_final_y, pen_x, pen_y);
	
	//printf("width = %d     ", pen_width);
	
loc64DB:			// new y
	counter = 0;
	circle_word = *circle_ptr;
	circle_ptr += 1;
loc64DF:			// new x
	if (pic.fCircle) pen_status |= 0x10;
	if (   ((pen_status & 0x10) == 0) || ( (binary_list[counter>>1] & circle_word) != 0)   )
	{	// == box plot, != circle plot
		if ( (pen_status & 0x20) == 0) goto loc6506;			// skip last column??  not sure

		temp8 = t % 2;
		t = t >> 1;
		if (temp8 != 0) 
			t = t ^ 0xB8;

		if ((t & 0x01) == 0) goto loc651b;
		if ((t & 0x02) == 0) goto loc651b;
	loc6506:
		//(push dx, ax, bx, si)
		AGI_Plot(pen_x, pen_y);
		//(pop si, bx, ax, dx)
	}
loc651b:
	pen_x++;
	counter += 3;	// needs to be 3 or the width will be wrong
	if (counter <= pen_width)
		goto loc64DF;
	pen_x = pen_final_x;
	pen_y++;
	if (pen_y != pen_final_y)
		goto loc64DB;
	return;
}

/*static void AGI_fPlotPattern(int x, int y)
{
	Uint16 *pCircle;
	int penSize, penMax, penWidth;
	Uint8 penStatus;
	Uint8 nPat;
	Uint16 wCircle;
	int counter;
	int x0, y0, x1, y1;
	Uint8 dummy;

	static Uint16 binary_list[] = {0x8000, 0x4000, 0x2000, 0x1000, 0x800, 0x400, 0x200, 0x100, 
		0x80, 0x40, 0x20, 0x10, 0x8, 0x4, 0x2, 0x1};

	static Uint8 circle_list[] = {0, 1, 4, 9, 16, 25, 37, 50};

	static Uint16 circle_data[] =
		{0x8000, 
		0xE000, 0xE000, 0xE000, 
		0x7000, 0xF800, 0x0F800, 0x0F800, 0x7000, 
		0x3800, 0x7C00, 0x0FE00, 0x0FE00, 0x0FE00, 0x7C00, 0x3800, 
		0x1C00, 0x7F00, 0x0FF80, 0x0FF80, 0x0FF80, 0x0FF80, 0x0FF80, 0x7F00, 0x1C00,
		0x0E00, 0x3F80, 0x7FC0, 0x7FC0, 0x0FFE0, 0x0FFE0, 0x0FFE0, 0x7FC0, 0x7FC0, 0x3F80, 0x1F00, 0x0E00,
		0x0F80, 0x3FE0, 0x7FF0, 0x7FF0, 0x0FFF8, 0x0FFF8, 0x0FFF8, 0x0FFF8, 0x0FFF8, 0x7FF0, 0x7FF0, 0x3FE0, 0x0F80,
		0x07C0, 0x1FF0, 0x3FF8, 0x7FFC, 0x7FFC, 0x0FFFE, 0x0FFFE, 0x0FFFE, 0x0FFFE, 0x0FFFE, 0x7FFC, 0x7FFC, 0x3FF8, 0x1FF0, 0x07C0};
	
	penStatus = pic.patCode;
	penSize = (penStatus & 0x07);
	pCircle = &circle_data[circle_list[penSize]];

	// set pen x coordinate
	x0 = (x * 2) - penSize;
	if (x0 < 0) x0 = 0;
	penMax = 320 - (penSize * 2);
	if (x0 > penMax) x0 = penMax;
	x0 /= 2;
	x1 = x + penSize;

	// set pen y coordinate
	y0 = y - penSize;
	if (y0 < 0) y0 = 0;
	penMax = 167 - (penSize * 2);
	if (y0 > penMax) y0 = penMax;
	y1 = y + penSize;

	// set pattern number
	nPat = pic.patNum | 0x01;

	// something
	penMax = (penSize * 2) + 1;
	//y1 += penMax;
	penWidth = penMax << 1;

	// loop

	for (y = y0; y <= y1; y++) {
		wCircle = *pCircle;
		pCircle += 1;
		counter = 0;
		for (x = x0; x <= x1; x++) {
			if (penStatus & 0x10)
				AGI_Plot(x, y);
			if (penStatus & 0x20) {
				if (wCircle & binary_list[counter >> 1]) {
					dummy = nPat % 2;
					nPat = nPat >> 1;
					if (dummy)
						nPat = nPat ^ 0xB8;
					if ((nPat & 0x01) || !(nPat & 0x02))
						nPat = nPat;
					else
						AGI_Plot(x, y);
				}
			}
			counter += 4;
			if (counter == penWidth) break;
		}
	}
}*/

static Uint8 circles[][15] = { /* agi circle bitmaps */
	{0x80},
	{0xfc},
	{0x5f, 0xf4},
	{0x66, 0xff, 0xf6, 0x60},
	{0x23, 0xbf, 0xff, 0xff, 0xee, 0x20},
	{0x31, 0xe7, 0x9e, 0xff, 0xff, 0xde, 0x79, 0xe3, 0x00},
	{0x38, 0xf9, 0xf3, 0xef, 0xff, 0xff, 0xff, 0xfe, 0xf9, 0xf3, 0xe3, 0x80},
	{0x18, 0x3c, 0x7e, 0x7e, 0x7e, 0xff, 0xff, 0xff, 0xff, 0xff, 0x7e, 0x7e, 0x7e, 0x3c, 0x18}
};

static Uint8 splatterMap[32] = { /* splatter brush bitmaps */
	0x20, 0x94, 0x02, 0x24, 0x90, 0x82, 0xa4, 0xa2,
	0x82, 0x09, 0x0a, 0x22, 0x12, 0x10, 0x42, 0x14,
	0x91, 0x4a, 0x91, 0x11, 0x08, 0x12, 0x25, 0x10,
	0x22, 0xa8, 0x14, 0x24, 0x00, 0x50, 0x24, 0x04
};

static Uint8 splatterStart[128] = { /* starting bit position */
	0x00, 0x18, 0x30, 0xc4, 0xdc, 0x65, 0xeb, 0x48,
	0x60, 0xbd, 0x89, 0x05, 0x0a, 0xf4, 0x7d, 0x7d,
	0x85, 0xb0, 0x8e, 0x95, 0x1f, 0x22, 0x0d, 0xdf,
	0x2a, 0x78, 0xd5, 0x73, 0x1c, 0xb4, 0x40, 0xa1,
	0xb9, 0x3c, 0xca, 0x58, 0x92, 0x34, 0xcc, 0xce,
	0xd7, 0x42, 0x90, 0x0f, 0x8b, 0x7f, 0x32, 0xed,
	0x5c, 0x9d, 0xc8, 0x99, 0xad, 0x4e, 0x56, 0xa6,
	0xf7, 0x68, 0xb7, 0x25, 0x82, 0x37, 0x3a, 0x51,
	0x69, 0x26, 0x38, 0x52, 0x9e, 0x9a, 0x4f, 0xa7,
	0x43, 0x10, 0x80, 0xee, 0x3d, 0x59, 0x35, 0xcf,
	0x79, 0x74, 0xb5, 0xa2, 0xb1, 0x96, 0x23, 0xe0,
	0xbe, 0x05, 0xf5, 0x6e, 0x19, 0xc5, 0x66, 0x49,
	0xf0, 0xd1, 0x54, 0xa9, 0x70, 0x4b, 0xa4, 0xe2,
	0xe6, 0xe5, 0xab, 0xe4, 0xd2, 0xaa, 0x4c, 0xe3,
	0x06, 0x6f, 0xc6, 0x4a, 0xa4, 0x75, 0x97, 0xe1
};

/*static void AGI_dPlotPattern(int x, int y)
{ 

	int circlePos = 0;
	Uint8 x1, y1, penSize, bitPos = splatterStart[pic.patNum];

	penSize = (pic.patCode & 7);

	if (x < ((penSize / 2) + 1))
		x = ((penSize / 2) + 1);
	else if (x > 320 - ((penSize / 2) + 1))
		x = 320 - ((penSize / 2) + 1);
	
	if (y < penSize)
		y = penSize;
	else if (y >= 168 - penSize)
		y = 168 - penSize;

	for (y1 = y - penSize; y1 <= y + penSize; y1++) {
		for (x1 = (Uint8)(x - (ceil((float)penSize / 2))); x1 <= (Uint8)(x + (floor((float)penSize / 2))); x1++) {
		//for (x1 = x - penSize; x1 <= x + penSize; x1++) {
			if (pic.patCode & 0x10) { // square
				AGI_PlotPatternPoint();
			}
			else { // circle
				if ((circles[pic.patCode & 7][circlePos >> 3] >> (7 - (circlePos & 7))) & 1) {
					AGI_PlotPatternPoint();
				}
				circlePos++;
			}
		}
	}
} */

/**************************************************************************
** AGI_PlotBrush
**
** Plots points and various brush patterns.
**************************************************************************/
static void AGI_PlotBrush(Uint8 **data)
{
	Uint8 x1, y1;

	for (;;) {
		if (pic.patCode & 0x20) {
			if ((pic.patNum = *((*data)++)) >= IDI_AGI_MIN) break;
			//pic.patNum = ((pic.patNum >> 1) & 0x7f);
		}

		if ((x1 = *((*data)++)) >= IDI_AGI_MIN) break;
		if ((y1 = *((*data)++)) >= IDI_AGI_MIN) break;

		AGI_PlotPattern(x1, y1);
	}

	(*data)--;
}

static void AGI_DrawPic_C64(Uint8 *buffer)
{
	int action;

	for (;;) {
		action = *(buffer++);
		switch (action) {
		case IDI_AGI_END:
			return;
		case 0xE0:
			AGI_DrawXcorner(&buffer); 
			break;
		case 0xE1:
			AGI_DrawYcorner(&buffer); 
			break;
		case 0xE2:
			AGI_DrawLineRel(&buffer); 
			break;
		case 0xE3:
			AGI_DrawLineAbs(&buffer); 
			break;
		case 0xE4:
			pic.colPic = *(buffer++);
			AGI_Fill(&buffer); 
			break;
		case 0xE5:
			pic.fDrawPic = true;
			break;
		case 0xE6:
			pic.patCode = *(buffer++);
			AGI_PlotBrush(&buffer);
			break;
		case 0xF0:
		case 0xF1:
		case 0xF2:
		case 0xF3:
		case 0xF4:
		case 0xF5:
		case 0xF6:
		case 0xF7:
		case 0xF8:
		case 0xF9:
		case 0xFA:
		case 0xFB:
		case 0xFC:
		case 0xFD:
		case 0xFE:
			pic.colPic = action - 0xF0;
			break;
		}
	}
}

static void AGI_DrawPic_v1(Uint8 *buffer)
{
	int action;

	for (;;) {
		action = *(buffer++);
		switch (action) {
		case IDI_AGI_END:
			return;
		case 0xF1: 
			pic.colPic = *(buffer++);
			pic.fDrawPic = true;
			pic.fDrawPri = false;
			break;
		case 0xF3: 
			pic.colPic = *(buffer++);
			pic.colPri = *(buffer++);
			pic.fDrawPic = true;
			pic.fDrawPri = true;
			break;
		case 0xF4: 
			AGI_DrawYcorner(&buffer); 
			break;
		case 0xF5: 
			AGI_DrawXcorner(&buffer); 
			break;
		case 0xF6: 
			AGI_DrawLineAbs(&buffer); 
			break;
		case 0xF7: 
			AGI_DrawLineRel(&buffer); 
			break;
		case 0xFA:
			pic.fDrawPic = false;
			pic.fDrawPri = true;
			AGI_DrawLineAbs(&buffer); 
			pic.fDrawPic = true;
			pic.fDrawPri = false;
			break;
		case 0xFB:
			AGI_DrawLineRel(&buffer); 
			break;
		case 0xFC:
			pic.colPic = *(buffer++);
			pic.colPri = *(buffer++);
			AGI_Fill(&buffer); 
			break;
		}
	}
}

static void AGI_DrawPic_v15(Uint8 *buffer)
{
	int action;

	for (;;) {
		action = *(buffer++);
		switch (action) {
		case IDI_AGI_END:
			return;
		case 0xF0: 
			break;
		case 0xF1: 
			pic.colPic = *(buffer++);
			pic.fDrawPic = true;
			break;
		case 0xF3: 
			break;
		case 0xF8: 
		case 0xF9: 
		case 0xFA: 
		case 0xFB: 
			AGI_DrawLineAbs(&buffer); 
			break;
		case 0xFE: 
			pic.colPic = *(buffer++);
			AGI_Fill(&buffer); 
			break;
		}
	}
}

static void AGI_DrawPic_v2(Uint8 *buffer)
{
	int action;

	for (;;) {
		action = *(buffer++);
		switch (action) {
		case IDI_AGI_END:
			return;
		case 0xF0: 
			pic.colPic = *(buffer++);
			pic.fDrawPic = true;
			break;
		case 0xF1: 
			//pic.fDrawPic = false;
			break;
		case 0xF2: 
			pic.colPri = *(buffer++);
			pic.fDrawPri = true;
			break;
		case 0xF3: 
			pic.fDrawPri = false;
			break;
		case 0xF4: 
			AGI_DrawYcorner(&buffer); 
			break;
		case 0xF5: 
			AGI_DrawXcorner(&buffer); 
			break;
		case 0xF6: 
			AGI_DrawLineAbs(&buffer); 
			break;
		case 0xF7: 
			AGI_DrawLineRel(&buffer); 
			break;
		case 0xF8: 
			AGI_Fill(&buffer); 
			break;
		case 0xF9: 
			pic.patCode = *(buffer++);
		case 0xFA: 
			AGI_PlotBrush(&buffer);
			break;
		}
		if (fStep) UpdateScreen();
	}
}

void DrawPictureAGI()
{
	for (int y = 0; y < IDI_AGI_PIC_MAX_HEIGHT; y++)
	{
		for (int x = 0; x < IDI_AGI_PIC_MAX_WIDTH; x++)
		{
			PutPixelAGI(x, y, pic.bmpPic[x][y]);
		}
	}
}

void DrawPictureBCG(Uint8 *buffer)
{
	const int w = 80;
	const int h = 170;
	Uint8 bitmap[h][w];

	memcpy(bitmap, buffer, sizeof(bitmap));

	LockScreen();

	for (int y = 0; y < h; y++)
	{
		for (int x = 0; x < w; x++)
		{
			Uint32 color  = BCGCOLOREVEN(BGCOLOR(bitmap[y][x]));
			Uint32 color2 = BCGCOLORODD (BGCOLOR(bitmap[y][x]));
			Uint32 color3 = BCGCOLOREVEN(FGCOLOR(bitmap[y][x]));
			Uint32 color4 = BCGCOLORODD (FGCOLOR(bitmap[y][x]));

			int vx = x * 4;
			int vy = y;

			PutPixel(vx, vy, color);
			PutPixel(vx + 1, vy, color2);
			PutPixel(vx + 2, vy, color3);
			PutPixel(vx + 3, vy, color4);
			PutPixel(vx, vy + 1, color);
			PutPixel(vx + 1, vy + 1, color2);
			PutPixel(vx + 2, vy + 1, color3);
			PutPixel(vx + 3, vy + 1, color4);
		}
	}

	UnlockScreen();
}

void ClearScreenAGI(int w, int h, int x0, int y0, int flags)
{
	int x, y;

	// set picture variables
	pic.w = w;
	pic.h = h;

	// clear both screens
	memset(pic.bmpPic, 0, sizeof(pic.bmpPic));
	memset(pic.bmpPri, 0, sizeof(pic.bmpPri));

	// draw picture drawing area
	for (y = y0; y < y0 + h; y++)
	{
		for (x = x0; x < x0 + w; x++)
		{
			pic.bmpPic[x][y] = IDC_AGI_PIC;
			pic.bmpPri[x][y] = IDC_AGI_PRI;
		}
	}

	DrawPictureAGI();

	// draw frame for old pictures
	if ((flags & IDF_AGI_PIC_V1) || (flags & IDF_AGI_PIC_V15))
	{
		for (x = 0; x < 160; x++)
		{
			pic.bmpPic[x][0] = 2;
			pic.bmpPic[x][167] = 2;
		}
		for (y = 0; y < 168; y++)
		{
			pic.bmpPic[0][y] = 2;
			pic.bmpPic[159][y] = 2;
		}
	}
}

void AGI_DrawPic(int x0, int y0, int flags, Uint8 *buffer)
{
	pic.x0 = x0;
	pic.y0 = y0;

	if (flags & IDF_AGI_STEP)
		fStep = true;

	if (flags & IDF_AGI_CIRCLE)
		pic.fCircle = true;

	if (flags & IDF_AGI_PIC_V1)
		AGI_DrawPic_v1(buffer);
	if (flags & IDF_AGI_PIC_V15)
		AGI_DrawPic_v15(buffer);
	if (flags & IDF_AGI_PIC_V2)
		AGI_DrawPic_v2(buffer);
	if (flags & IDF_AGI_PIC_C64)
		AGI_DrawPic_C64(buffer);

	fStep = false;
	pic.fCircle = false;
}
