/*
 *  Copyright (C) 2003-2004 Curt Coder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <string>
#include <SDL.h>
#include "agipic.h"
#include "driver.h"
#include "fileio.h"
#include "font.h"
#include "gfx.h"
#include "selgame.h"
#include "ui.h"
#include "gelfling.h"

static int moves = 0;
static int hiscore = 66;
static int room = 1;

// File Operations

static void Gelfling_ReadRoomText(int iRoom, char *szMsg, char *szMenu, int *nSel)
{
	char szFile[256] = {0};
	char szTemp[41] = {0};
	char szLine[41] = {0};
	int pc;
	int i;
	Uint8 *buffer = new Uint8[IDI_GEL_LEN_TEXT];

	// read data
	sprintf(szFile, IDS_GEL_PATH, GetGamePath(), IDS_GEL_FILE_2);
	ReadFileOfs(szFile, iRoom * IDI_GEL_LEN_TEXT, buffer, IDI_GEL_LEN_TEXT);

	// replace 0x8D with 0x00 so that strcpy works correctly
	for (i = 0; i < IDI_GEL_LEN_TEXT; i++)
		if (buffer[i] == 0x8D) buffer[i] = 0x00;

	// extract message
	memset(szMsg, 0, sizeof(szMsg));
	memset(szMenu, 0, sizeof(szMenu));
	pc = 1;
	strcpy(szMsg, (char *)buffer + pc);
	pc += strlen(szMsg) + 2;

	// extract menu
	strcpy(szTemp, (char *)buffer + pc);
	pc += strlen(szTemp) + 2;
	strcat(szMenu, szTemp);
	strcat(szMenu, "\n");

	for (i = 1; i < 4; i++) {
		strcpy(szTemp, (char *)buffer + pc);
		pc += strlen(szTemp) + 2;
		if (!strlen(szTemp)) break;
		sprintf(szLine, " %d. %s\n", i, szTemp);
		strcat(szMenu, szLine);
	}

	*nSel = i - 1;

	delete [] buffer;
}

static void Gelfling_PrintBmessage(int iMsg)
{
	FILE *infile;
	char szFile[256] = {0};
	char szMsg[IDI_GEL_MAX_LEN_MSG] = {0};
	bool diskerror = true;
	GEL_BMESSAGE_HDR hdr;

	// construct filename
	sprintf(szFile, IDS_GEL_PATH, GetGamePath(), IDS_GEL_FILE_0);

	// open file
	while (diskerror) {
		infile = fopen(szFile, "rb");
		if (infile == NULL) {
			PrintMsg(IDS_GEL_DISK);
		}
		else
			diskerror = false;
	}

	// clear buffer and read data
	memset(szMsg, 0, sizeof(szMsg));
	fread(&hdr, 1, sizeof(GEL_BMESSAGE_HDR), infile);
	fseek(infile, hdr.offset[iMsg - 1] - IDI_GEL_OFS_BMESSAGE, SEEK_SET);
	fgets(szMsg, IDI_GEL_MAX_LEN_MSG, infile);
	
	fclose(infile);

	PrintMsg(szMsg);
}

static void Gelfling_PrintCongrats(int iMsg)
{
	FILE *infile;
	char szFile[256] = {0};
	char szMsg[IDI_GEL_MAX_LEN_MSG] = {0};
	bool diskerror = true;
	GEL_CONGRATS_HDR hdr;

	// construct filename
	sprintf(szFile, IDS_GEL_PATH, GetGamePath(), IDS_GEL_FILE_1);

	// open file
	while (diskerror) {
		infile = fopen(szFile, "rb");
		if (infile == NULL) {
			PrintMsg(IDS_GEL_DISK);
		}
		else
			diskerror = false;
	}

	// clear buffer and read data
	memset(szMsg, 0, sizeof(szMsg));
	fread(&hdr, 1, sizeof(GEL_CONGRATS_HDR), infile);
	fseek(infile, hdr.offset[iMsg - 1] - IDI_GEL_OFS_CONGRATS, SEEK_SET);
	fgets(szMsg, IDI_GEL_MAX_LEN_MSG, infile);

	fclose(infile);
	
	PrintMsg(szMsg);
}

// User Interface

static void Gelfling_GetMenuSel(char *szMsg, char *szMenu, int *iSel, int nSel)
{
	SDL_Event event;
	int x, y;
	
	if (strlen(szMsg)) {
		PrintMsg(szMsg);
	}

	DrawMenu(szMenu, *iSel);

	for (;;) {
		while (SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_QUIT:
				exit(0);
			case SDL_MOUSEMOTION:
				x = event.motion.x / GetFontWidth();
				y = event.motion.y / GetFontHeight();
				GetMouseMenuSel(iSel, nSel, x, y);
				DrawMenu(szMenu, *iSel);
				break;
			case SDL_MOUSEBUTTONUP:
				switch(event.button.button) {
				case SDL_BUTTON_LEFT:
					return;
				case SDL_BUTTON_RIGHT:
					event.type = SDL_KEYDOWN;
					event.key.keysym.sym = SDLK_LEFT;
					SDL_PushEvent(&event);
					break;
				}
				break;
			case SDL_KEYDOWN:
				switch (event.key.keysym.sym) {
				case SDLK_BACKSPACE:
				case SDLK_LEFT:
					if (strlen(szMsg)) {
						PrintMsg(szMsg);
						DrawMenu(szMenu, *iSel);
					} else {
						*iSel = IDI_GEL_SEL_BACK;
						return;
					}
					break;
				case SDLK_SPACE:
					*iSel += 1;
					if (*iSel == nSel) *iSel = IDI_GEL_SEL_OPTION_1;
					DrawMenu(szMenu, *iSel);
					break;
				case SDLK_RETURN:
				case SDLK_KP_ENTER:
					return;
				}
				break;
			}
		}
	}
}

// Graphics

static void Gelfling_DrawRoomPic(int iRoom)
{
	char szFile[256] = {0};
	Uint8 *buffer = (Uint8 *)malloc(4096);

	// construct filename
	sprintf(szFile, IDS_GEL_PATH_PIC, GetGamePath(), iRoom);

	// read picture file into buffer
	ReadFile(szFile, buffer);

	// draw picture
	ClearScreenAGI(IDI_GEL_PIC_WIDTH, IDI_GEL_PIC_HEIGHT, IDI_GEL_PIC_X0, IDI_GEL_PIC_Y0, IDI_GEL_PIC_FLAGS);
	AGI_DrawPic(IDI_GEL_PIC_X0, IDI_GEL_PIC_Y0, IDI_GEL_PIC_FLAGS, buffer);
	UpdateScreen();

	free(buffer);
}

// Game Logic

static void Gelfling_Credits()
{
	ClearScreen(IDA_DEFAULT);
	DrawStr(0, 0, IDS_GEL_CREDITS_0);
	DrawStr(2, 9, IDS_GEL_CREDITS_1);
	DrawStr(3, 9, IDS_GEL_CREDITS_2);
	DrawStr(4, 9, IDS_GEL_CREDITS_3);
	DrawStr(7, 15, IDS_GEL_CREDITS_4);
	DrawStr(8, 19, IDS_GEL_CREDITS_5);
	DrawStr(9, 11, IDS_GEL_CREDITS_6);
	DrawStr(12, 4, IDS_GEL_CREDITS_7);
	DrawStr(13, 11, IDS_GEL_CREDITS_8);
	DrawStr(14, 11, IDS_GEL_CREDITS_9);
	DrawStr(16, 13, IDS_GEL_CREDITS_10);
	DrawStr(17, 9, IDS_GEL_CREDITS_11);
	DrawStr(18, 7, IDS_GEL_CREDITS_12);
	DrawStr(20, 6, IDS_GEL_CREDITS_13);
	DrawStr(21, 10, IDS_GEL_CREDITS_14);
	DrawStr(22, 0, IDS_GEL_ANY_KEY);
	UpdateScreen();
	WaitAnyKey();
}

static void Gelfling_GameOver()
{
	char szMoves[IDI_GEL_MAX_LEN_MSG] = {0};

	sprintf(szMoves, IDS_GEL_GAME_OVER_1, moves, hiscore);

	PrintMsg(IDS_GEL_GAME_OVER_0);
	PrintMsg(szMoves);
	if (moves < hiscore) {
		PrintMsg(IDS_GEL_GAME_OVER_2);
		PrintMsg(IDS_GEL_GAME_OVER_3);
	}
	PrintMsg(IDS_GEL_GAME_OVER_4);
}

static void Gelfling_Tutorial()
{
	bool done = false, done2 = false;
	int iSel;

	while (!done) {
		PrintMsg(IDS_GEL_TUTORIAL_0);
		PrintMsg(IDS_GEL_TUTORIAL_1);

		iSel = 1;
		Gelfling_GetMenuSel("", IDS_GEL_TUTORIAL_2, &iSel, 3);
		
		done2 = false;
		while (!done2) {
			iSel = 0;
			Gelfling_GetMenuSel("", IDS_GEL_TUTORIAL_3, &iSel, 3);
			switch(iSel) {
			case IDI_GEL_SEL_BACK:
				done2 = true;
				break;
			case IDI_GEL_SEL_OPTION_1:
				PrintMsg(IDS_GEL_TUTORIAL_4);
				break;
			case IDI_GEL_SEL_OPTION_2:
				PrintMsg(IDS_GEL_TUTORIAL_5);
				break;
			case IDI_GEL_SEL_OPTION_3:
				done = true;
				done2 = true;
				break;
			}
		}
	}
}

static void Gelfling_Intro()
{
	PrintStr(IDS_GEL_INTRO);

	if (!GetSelYN()) {
		Gelfling_Tutorial();
	}

	Gelfling_Credits();
}

static void Gelfling_GameLoop()
{
	int iSel;
	int nSel;
	bool done = false;
	char szMsg[IDI_GEL_MAX_LEN_MSG];
	char szMenu[IDI_GEL_MAX_LEN_MSG];
	
	while (!done) {
		Gelfling_DrawRoomPic(room);
		Gelfling_ReadRoomText(room, szMsg, szMenu, &nSel);

		iSel = 0;
		Gelfling_GetMenuSel(szMsg, szMenu, &iSel, nSel);
		moves++;

		switch(iSel) {
		case IDI_GEL_SEL_OPTION_1:
			room = 1;
			break;
		case IDI_GEL_SEL_OPTION_2:
			room = 2;
			break;
		case IDI_GEL_SEL_OPTION_3:
			room = 3;
			break;
		}
	}
}

// Init

static void Gelfling_InitEngine()
{
	SetScreenPar(320, 200, (char*)ibm_fontdata);
	SetMenuPars(20, 21, "| |");
}

void Gelfling_Init()
{
	Gelfling_InitEngine();
	Gelfling_Intro();
	Gelfling_GameLoop();
	Gelfling_GameOver();
}

// Driver

GAME (gelfling, 1984, "Sierra On-Line", "Gelfling Adventure", GAME_NOT_WORKING | GAME_NO_SOUND, Gelfling_Init)
