/*
 *  Copyright (C) 2003-2004 Curt Coder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

// 
// Mickey's Space Adventure
// Copyright  1986 The Walt Disney Company.
//    TM designates trademark of Sierra.
//          Licensed to Tandy Corp.
// 

#include <string>
#include <SDL.h>
#include "agipic.h"
#include "driver.h"
#include "fileio.h"
#include "font.h"
#include "gfx.h"
#include "selgame.h"
#include "sound.h"
#include "ui.h"
#include "mickey.h"

static MSA_GAME game;

void Mickey_PrintExeStr(int ofs);

// File Operations

static int Mickey_GetDat(int iRoom)
{
	if (((iRoom > 0) && (iRoom < 24)) || iRoom == 154 || iRoom == 155) return IDI_MSA_PLANET_EARTH;
	if ((iRoom >= 30) && (iRoom <= 39)) return IDI_MSA_PLANET_VENUS;
	if ((iRoom >= 40) && (iRoom <= 69)) return IDI_MSA_PLANET_NEPTUNE;
	if ((iRoom >= 70) && (iRoom <= 82)) return IDI_MSA_PLANET_MERCURY;
	if ((iRoom >= 83) && (iRoom <= 92)) return IDI_MSA_PLANET_SATURN;
	if ((iRoom >= 93) && (iRoom <= 103)) return IDI_MSA_PLANET_PLUTO;
	if ((iRoom >= 106) && (iRoom <= 120)) return IDI_MSA_PLANET_JUPITER;
	if ((iRoom >= 121) && (iRoom <= 132)) return IDI_MSA_PLANET_MARS;
	if ((iRoom >= 133) && (iRoom <= 145)) return IDI_MSA_PLANET_URANUS;
	return IDI_MSA_PLANET_SPACESHIP;
}

static void Mickey_ReadExe(int ofs, Uint8 *buffer, long buflen)
{
	char szFile[256] = {0};

	sprintf(szFile, IDS_MSA_PATH_EXE, GetGamePath());
	ReadFileOfs(szFile, ofs, buffer, buflen);
}

static FILE* Mickey_ReadDatHeader(int iDat, MSA_DAT_HEADER *hdr)
{
	FILE *infile;
	bool diskerror = true;
	char szFile[256] = {0};

	// construct filename
	sprintf(szFile, IDS_MSA_PATH_DAT, GetGamePath(), IDS_MSA_NAME_DAT[iDat]);
	
	// open file
	while (diskerror) {
		infile = fopen(szFile, "rb");
		if (infile == NULL) {
			Mickey_PrintExeStr(IDO_MSA_CHECK_DISK_DRIVE);
			WaitAnyKey();
		} else {
			diskerror = false;
		}
	}

	// clear buffer and read data
	memset(hdr, 0, sizeof(MSA_DAT_HEADER));
	fread(hdr, 1, sizeof(MSA_DAT_HEADER), infile);

	return infile;
}

static void Mickey_GetDatFileName(int iRoom, char *szFile)
{
	sprintf(szFile, IDS_MSA_PATH_DAT, GetGamePath(), IDS_MSA_NAME_DAT[Mickey_GetDat(iRoom)]);
}

static void Mickey_ReadDatHdr(char *szFile, MSA_DAT_HEADER *hdr)
{
	Uint8 *buffer = new Uint8[sizeof(MSA_DAT_HEADER)];

	ReadFileOfs(szFile, 0, buffer, sizeof(MSA_DAT_HEADER));
	memcpy(hdr, buffer, sizeof(MSA_DAT_HEADER));
}

static void Mickey_ReadDesc(int iRoom, char *buffer, long buflen)
{
	MSA_DAT_HEADER hdr;
	char szFile[256] = {0};
	
	Mickey_GetDatFileName(iRoom, szFile);
	Mickey_ReadDatHdr(szFile, &hdr);
	ReadFileOfs(szFile, hdr.ofsDesc[iRoom - 1] + IDI_MSA_OFS_DAT, (Uint8 *)buffer, buflen);
}

static void Mickey_ReadMenu(int iRoom, char *buffer)
{
	MSA_DAT_HEADER hdr;
	char szFile[256] = {0};

	Mickey_GetDatFileName(iRoom, szFile);
	Mickey_ReadDatHdr(szFile, &hdr);
	ReadFileOfs(szFile, hdr.ofsRoom[iRoom - 1] + IDI_MSA_OFS_DAT, (Uint8 *)buffer, sizeof(MSA_MENU));
}

static void Mickey_ReadDatStr(int iDat, int iStr, char *buffer, long buflen)
{
	MSA_DAT_HEADER hdr;
	char szFile[256] = {0};
	
	sprintf(szFile, IDS_MSA_PATH_DAT, GetGamePath(), IDS_MSA_NAME_DAT[iDat]);
	Mickey_ReadDatHdr(szFile, &hdr);
	ReadFileOfs(szFile, hdr.ofsStr[iStr] + IDI_MSA_OFS_DAT, (Uint8 *)buffer, buflen);
}

static void Mickey_ReadOfsData(int offset, int iItem, Uint8 *buffer, long buflen)
{
	Uint16 ofs[256];

	Mickey_ReadExe(offset, buffer, buflen);
	memcpy(ofs, buffer, sizeof(ofs));
	Mickey_ReadExe(ofs[iItem] + IDI_MSA_OFS_EXE, buffer, buflen);
}

// User Interface

static bool Mickey_WaitAnyKey()
{
	SDL_Event event;

	for (;;) {
		while (SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_QUIT:
				exit(0);
			case SDL_MOUSEBUTTONUP:
				switch(event.button.button) {
				case SDL_BUTTON_LEFT:
					return true;
				case SDL_BUTTON_RIGHT:
					return false;
				}
				break;
			case SDL_KEYDOWN:
				switch (event.key.keysym.sym) {
				case SDLK_ESCAPE:
					return false;
				default:
					return true;
				}
				break;
			}
		}
	}
}

static void Mickey_WaitAnyKeyAnim()
{
	SDL_Event event;

	for (;;) {
		while (SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_QUIT:
				exit(0);
			case SDL_MOUSEBUTTONUP:
				return;
			case SDL_KEYDOWN:
				return;
			}
		}
		Mickey_Animate();
		UpdateScreen();
	}
}

static bool Mickey_GetSelYN(int ofsPrompt, bool fErrorMsg)
{
	SDL_Event event;

	Mickey_PrintExeStr(ofsPrompt);

	for (;;) {
		while (SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_QUIT:
				exit(0);
			case SDL_MOUSEBUTTONUP:
				switch(event.button.button) {
				case SDL_BUTTON_LEFT:
					return true;
				case SDL_BUTTON_RIGHT:
					return false;
				}
				break;
			case SDL_KEYDOWN:
				switch (event.key.keysym.sym) {
				case SDLK_y:
					return true;
				case SDLK_n:
					return false;
				default:
					if (fErrorMsg) {
						Mickey_PrintExeStr(IDO_MSA_PRESS_YES_OR_NO);
						WaitAnyKey();
						Mickey_PrintExeStr(ofsPrompt);
					}
					break;
				}
				break;
			}
		}
	}
}

static int Mickey_GetSel1to9(int ofsPrompt)
{
	SDL_Event event;

	Mickey_PrintExeStr(ofsPrompt);

	for (;;) {
		while (SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_QUIT:
				exit(0);
			case SDL_MOUSEBUTTONUP:
				switch(event.button.button) {
				case SDL_BUTTON_RIGHT:
					return 0;
				}
				break;
			case SDL_KEYDOWN:
				switch (event.key.keysym.sym) {
				case SDLK_ESCAPE:
					return 0;
				case SDLK_1:
				case SDLK_2:
				case SDLK_3:
				case SDLK_4:
				case SDLK_5:
				case SDLK_6:
				case SDLK_7:
				case SDLK_8:
				case SDLK_9:
					return event.key.keysym.sym - SDLK_1 + 1;
				default:
					Mickey_PrintExeStr(IDO_MSA_PRESS_1_TO_9);
					if (!Mickey_WaitAnyKey()) return 0;
					Mickey_PrintExeStr(ofsPrompt);
					break;
				}
				break;
			}
		}
	}
}

static void Mickey_PrintStr(char *buffer)
{
	int pc = 1;
	int nRows, iCol, iRow;
	
	nRows = *buffer + IDI_MSA_ROW_MENU_0;

	ClearTextArea();

	for (iRow = IDI_MSA_ROW_MENU_0; iRow < nRows; iRow++) {
		iCol = *(buffer + pc++);
		DrawStr(iRow, iCol, buffer + pc);
		pc += strlen(buffer + pc) + 1;
	}
}

static void Mickey_PrintExeStr(int ofs)
{
	Uint8 buffer[256] = {0};

	if (!ofs) return;

	Mickey_ReadExe(ofs, buffer, sizeof(buffer));
	Mickey_PrintStr((char *)buffer);
	UpdateScreen();
}

static void Mickey_PrintExeMsg(int ofs)
{
	if (!ofs) return;
	Mickey_PrintExeStr(ofs);
	Mickey_WaitAnyKeyAnim();
}

static void Mickey_PrintDatStr(int iDat, int iStr)
{
	char *buffer = (char *)malloc(256);
	Mickey_ReadDatStr(iDat, iStr, buffer, 256);
	Mickey_PrintStr(buffer);
	free(buffer);
}

static void Mickey_PrintDesc(int iRoom)
{
	char *buffer = (char *)malloc(256);
	Mickey_ReadDesc(iRoom, buffer, 256);
	Mickey_PrintStr(buffer);
	UpdateScreen();
	free(buffer);
}

static void Mickey_DrawMenu(MSA_MENU menu, int sel0, int sel1)
{
	int iWord;
	int iRow;
	int sel;
	Uint8 attr;

	// draw menu

	ClearTextArea();

	for (iRow = 0; iRow < 2; iRow++) {
		for (iWord = 0; iWord < menu.row[iRow].count; iWord++) {
			if (iRow)
				sel = sel1; 
			else 
				sel = sel0;

			if (iWord == sel) 
				attr = IDA_DEFAULT_REV; 
			else 
				attr = IDA_DEFAULT;

			DrawStrAttr(IDI_MSA_ROW_MENU_0 + iRow, menu.row[iRow].entry[iWord].x0, 
				attr, (char *)menu.row[iRow].entry[iWord].szText);
		}
	}

	UpdateScreen();
}

static void Mickey_GetMouseMenuSelRow(MSA_MENU menu, int *sel0, int *sel1, int iRow, int x, int y)
{
	int iWord;
	int *sel;

	switch(iRow) {
	case 0:
		if (y != IDI_MSA_ROW_MENU_0) return;
		sel = sel0;
		break;
	case 1:
		if (y != IDI_MSA_ROW_MENU_1) return;
		sel = sel1;
		break;
	}

	for (iWord = 0; iWord < menu.row[iRow].count; iWord++) {
		if ((x >= menu.row[iRow].entry[iWord].x0) && 
			(x < (int)(menu.row[iRow].entry[iWord].x0 + 
			strlen((char *)menu.row[iRow].entry[iWord].szText)))) {
				*sel = iWord;
				break;
		}
	}
}

static bool Mickey_GetMenuSelRow(MSA_MENU menu, int *sel0, int *sel1, int iRow)
{
	SDL_Event event;
	int *sel;
	int nWords;
	int x, y;

	switch(iRow) {
	case 0:
		sel = sel0;
		break;
	case 1:
		sel = sel1;
		break;
	}
	nWords = menu.row[iRow].count;

	Mickey_DrawMenu(menu, *sel0, *sel1);

	for (;;) {
		while (SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_QUIT:
				exit(0);
			case SDL_MOUSEMOTION:
				if (iRow < 2) {
					x = event.motion.x / GetFontWidth();
					y = event.motion.y / GetFontHeight();
					Mickey_GetMouseMenuSelRow(menu, sel0, sel1, iRow, x, y);
					Mickey_DrawMenu(menu, *sel0, *sel1);
				}
				break;
			case SDL_MOUSEBUTTONUP:
				switch(event.button.button) {
				case SDL_BUTTON_LEFT:
					return true;
				case SDL_BUTTON_MIDDLE:
					Mickey_Inventory();
					Mickey_DrawRoom();
					*sel0 = 0; *sel1 = -1;
					return false;
				case SDL_BUTTON_RIGHT:
					*sel0 = 0; *sel1 = -1; 
					return false;
				case SDL_BUTTON_WHEELUP:
					event.type = SDL_KEYDOWN;
					event.key.keysym.sym = SDLK_LEFT;
					SDL_PushEvent(&event);
					break;
				case SDL_BUTTON_WHEELDOWN:
					event.type = SDL_KEYDOWN;
					event.key.keysym.sym = SDLK_RIGHT;
					SDL_PushEvent(&event);
					break;
				}
				break;
			case SDL_KEYDOWN:
				switch (event.key.keysym.sym) {
				case SDLK_F11:
					FlipCGA();
					Mickey_DrawRoom();
					Mickey_DrawMenu(menu, *sel0, *sel1);
					break;
				case SDLK_F12:
					FlipDblSize();
					Mickey_DrawRoom();
					Mickey_DrawMenu(menu, *sel0, *sel1);
					break;
				case SDLK_2:
					Mickey_Hidden();
					break;
				case SDLK_8:
					if (event.key.keysym.mod & KMOD_CTRL) {
						*sel0 = 0; *sel1 = -1; return false;
					}
					break;
				case SDLK_ESCAPE:
					*sel0 = 0; *sel1 = -1; return false;
				case SDLK_s:
					FlipSound();
					break;
				case SDLK_c:
					Mickey_Inventory();
					Mickey_DrawRoom();
					*sel0 = 0; *sel1 = -1; return false;
				case SDLK_b:
					Mickey_PrintRoomDesc();
					Mickey_DrawMenu(menu, *sel0, *sel1);
					*sel0 = 0; *sel1 = -1; return false;
				case SDLK_LEFT:
				case SDLK_KP4:
				case SDLK_4:
					if (iRow < 2) {
						*sel -= 1;
						if (*sel < 0) *sel = nWords - 1;
						Mickey_DrawMenu(menu, *sel0, *sel1);
					}
					break;
				case SDLK_RIGHT:
				case SDLK_SPACE:
				case SDLK_KP6:
				case SDLK_6:
					if (iRow < 2) {
						*sel += 1;
						if (*sel == nWords) *sel = 0;
						Mickey_DrawMenu(menu, *sel0, *sel1);
					}
					break;
				case SDLK_RETURN:
				case SDLK_KP_ENTER:
					return true;
				}
				break;
			}
			Mickey_Animate();
			Mickey_DrawMenu(menu, *sel0, *sel1);
		}
		Mickey_Animate();
		Mickey_DrawMenu(menu, *sel0, *sel1);
	}
}

static void Mickey_GetMenuSel(char *buffer, int *sel0, int *sel1)
{
	MSA_MENU menu;

	memcpy(&menu, buffer, sizeof(MSA_MENU));

	*sel0 = 0;
	*sel1 = -1;

	for (;;) {
		for (;;) {
			if (Mickey_GetMenuSelRow(menu, sel0, sel1, 0)) {
				*sel1 = 0;
				if (Mickey_GetMenuSelRow(menu, sel0, sel1, 1)) {
					break;
				}
			}
		}
		if (Mickey_GetMenuSelRow(menu, sel0, sel1, 2)) {
			break;
		}
	}
}

static void Mickey_CenterMenu(MSA_MENU *menu)
{
	int iWord;
	int iRow;
	int w, x;

	for (iRow = 0; iRow < 2; iRow++) {
		w = 0;
		for (iWord = 0; iWord < menu->row[iRow].count; iWord++) {
			w += strlen((char *)menu->row[iRow].entry[iWord].szText);
		}
		w += menu->row[iRow].count - 1;
		x = (40 - w) / 2;	// FIX
		for (iWord = 0; iWord < menu->row[iRow].count; iWord++) {
			menu->row[iRow].entry[iWord].x0 = x;
			x += strlen((char *)menu->row[iRow].entry[iWord].szText) + 1;
		}
	}
}

static void Mickey_PatchMenu(MSA_MENU *menu)
{
	Uint8 buffer[512];
	Uint8 menubuf[sizeof(MSA_MENU)];
	int nPatches;
	int pBuf = 0;

	// change planet name in ship airlock menu
	if (game.iRoom == IDI_MSA_PIC_SHIP_AIRLOCK) {
		strcpy((char *)menu->row[1].entry[2].szText, IDS_MSA_NAME_PLANET[game.iPlanet]);
	}

	// exit if fix unnecessary
	if (!game.iRmMenu[game.iRoom]) {
		Mickey_CenterMenu(menu);
		return;
	}

	// copy menu to menubuf
	memcpy(menubuf, menu, sizeof(menubuf));

	// read patches
	Mickey_ReadOfsData(
		IDOFS_MSA_MENU_PATCHES, 
		game.nRmMenu[game.iRoom] + game.iRmMenu[game.iRoom] - 1,
		buffer, sizeof(buffer)
	);

	// get number of patches
	nPatches = buffer[pBuf++];

	// patch menubuf
	for (int iPatch = 0; iPatch < nPatches; iPatch++) {
		if (buffer[pBuf] > sizeof(menubuf)) {
			// patch address out of bounds
		}
		menubuf[buffer[pBuf]] = buffer[pBuf + 1];
		pBuf += 2;
	}

	// copy menubuf back to menu
	memcpy(menu, menubuf, sizeof(MSA_MENU));

	// center menu
	Mickey_CenterMenu(menu);
}

static void Mickey_PrintDatString(int iStr)
{
	Mickey_PrintDatStr(Mickey_GetDat(game.iRoom), iStr);
}

static void Mickey_PrintDatMessage(int iStr)
{
	Mickey_PrintDatString(iStr);
	UpdateScreen();
	Mickey_WaitAnyKeyAnim();
}

// Sound

static void Mickey_PlayNote(MSA_SND_NOTE note)
{
	if (!note.counter) {
		PlayNote(1, 0, note.length / IDI_SND_TIMER_RESOLUTION);
	} else {
		PlayNote(1, IDI_SND_OSCILLATOR_FREQUENCY / note.counter, 
			note.length / IDI_SND_TIMER_RESOLUTION / IDI_SND_PITCH);
	}
}

static void Mickey_PlaySound(ENUM_MSA_SOUND iSound)
{
	if (!GetSound()) return;

	SDL_Event event;
	MSA_SND_NOTE note;
	Uint8 *buffer = new Uint8[1024];
	int pBuf = 1;

	switch(iSound) {
	case IDI_MSA_SND_XL30:
		for (int iNote = 0; iNote < 6; iNote++) {
			note.counter = Rnd(59600) + 59;
			note.length = 4;
			Mickey_PlayNote(note);
		}
		break;
	default:
		Mickey_ReadOfsData(IDOFS_MSA_SOUND_DATA, iSound, buffer, 1024);

		for (;;) {
			memcpy(&note, buffer + pBuf, sizeof(note));
			if (!note.counter && !note.length) break;
			Mickey_PlayNote(note);
			pBuf += 3;

			if (iSound == IDI_MSA_SND_THEME) {
				while (SDL_PollEvent(&event)) {
					switch(event.type) {
					case SDL_QUIT:
						exit(0);
					case SDL_MOUSEBUTTONUP:
					case SDL_KEYDOWN:
						delete [] buffer;
						return;
					}
				}
			}
		}

		break;
	}

	delete [] buffer;
}

static void Mickey_Debug()
{
	char szLine[41] = {0};

	ClearScreen(IDA_DEFAULT);

	sprintf(szLine, IDS_MSA_DEBUG_ROOM, game.iRoom);
	DrawStr(5, 10, szLine);

	if (game.iRoom < IDI_MSA_MAX_PIC_ROOM) {
		if (game.iRmObj[game.iRoom] != IDI_MSA_OBJECT_NONE) {
			sprintf(szLine, IDS_MSA_DEBUG_OBJ, game.iRmObj[game.iRoom]);
			DrawStr(7, 10, szLine);
		}
	} else {
		sprintf(szLine, IDS_MSA_DEBUG_OBJ, 32);
		DrawStr(7, 10, szLine);
	}
	UpdateScreen();
}

// Graphics

static void Mickey_DrawObj(ENUM_MSA_OBJECT iObj, int x0, int y0)
{
	Uint8 *buffer = new Uint8[4096];
	char szFile[255] = {0};

	sprintf(szFile, IDS_MSA_PATH_OBJ, GetGamePath(), IDS_MSA_NAME_OBJ[iObj]);
	ReadFile(szFile, buffer);

	if (iObj == IDI_MSA_OBJECT_CRYSTAL) {
		AGI_DrawPic(IDI_MSA_PIC_X0 + x0, IDI_MSA_PIC_Y0 + y0, IDF_AGI_PIC_V2 | IDF_AGI_STEP, buffer);
	} else {
		AGI_DrawPic(IDI_MSA_PIC_X0 + x0, IDI_MSA_PIC_Y0 + y0, IDF_AGI_PIC_V2, buffer);
	}

	delete [] buffer;
}

static void Mickey_DrawPic(int iPic)
{
	int w = IDI_MSA_PIC_WIDTH, h = IDI_MSA_PIC_HEIGHT;
	int flags = IDF_AGI_PIC_V2;

	if (iPic == IDI_MSA_PIC_STAR_MAP) {
		w++; 
		h++;
		flags |= IDF_AGI_CIRCLE;
	}

	ClearScreenAGI(w, h, IDI_MSA_PIC_X0, IDI_MSA_PIC_Y0, flags);

	if (iPic) {
		Uint8 *buffer = new Uint8[4096];
		char szFile[255] = {0};

		sprintf(szFile, IDS_MSA_PATH_PIC, GetGamePath(), iPic);
		ReadFile(szFile, buffer);
		AGI_DrawPic(IDI_MSA_PIC_X0, IDI_MSA_PIC_Y0, flags, buffer);
		delete [] buffer;
	}
}

static void Mickey_DrawRoomPicture()
{
	if (GetDebug()) {
		Mickey_DrawPic(0);
		Mickey_Debug();
	} else {
		if (game.iRoom == IDI_MSA_PIC_TITLE) {
			Mickey_DrawPic(IDI_MSA_PIC_TITLE);
		} else {
			Mickey_DrawPic(game.iRmPic[game.iRoom]);
		}
	}
}

static void Mickey_DrawRoomObjects()
{
	if (game.iRoom >= IDI_MSA_MAX_PIC_ROOM) return;

	Uint8 buffer[256];
	int pBuf = 0;
	int nObjs;

	// draw ship control room window

	if (game.iRoom == IDI_MSA_PIC_SHIP_CONTROLS) {
		if (game.fFlying) {
			Mickey_DrawObj(IDI_MSA_OBJECT_W_SPACE, 0, 0);
		} else {
			Mickey_DrawObj((ENUM_MSA_OBJECT)(IDI_MSA_OBJECT_W_EARTH + game.iPlanet), 0, 1);
		}
	}

	// draw objects

	if (game.iRmObj[game.iRoom] != IDI_MSA_OBJECT_NONE) {
		Mickey_ReadOfsData(IDO_MSA_ROOM_OBJECT_XY_OFFSETS, 
			game.iRmObj[game.iRoom], buffer, sizeof(buffer));

		nObjs = buffer[pBuf++];

		for (int iObj = 0; iObj < nObjs; iObj++) {
			Mickey_DrawObj((ENUM_MSA_OBJECT)buffer[pBuf], buffer[pBuf + 1], buffer[pBuf + 2]);
			pBuf += 3;
		}
	}
}

static void Mickey_DrawRoomAnimation()
{
	Uint8 objLight[] = {
		0xF0, 1, 0xF9, 2, 53, 45, 0xFF
	};

	switch(game.iRoom) {
	case IDI_MSA_PIC_EARTH_SHIP:
	case IDI_MSA_PIC_VENUS_SHIP:
	case IDI_MSA_PIC_NEPTUNE_SHIP:
	case IDI_MSA_PIC_MERCURY_SHIP:
	case IDI_MSA_PIC_SATURN_SHIP:
	case IDI_MSA_PIC_PLUTO_SHIP:
	case IDI_MSA_PIC_JUPITER_SHIP:
	case IDI_MSA_PIC_MARS_SHIP:
	case IDI_MSA_PIC_URANUS_SHIP:
	case IDI_MSA_PIC_SHIP_VENUS:
	case IDI_MSA_PIC_SHIP_NEPTUNE:
	case IDI_MSA_PIC_SHIP_MERCURY:
	case IDI_MSA_PIC_SHIP_SATURN:
	case IDI_MSA_PIC_SHIP_PLUTO:
	case IDI_MSA_PIC_SHIP_JUPITER:
	case IDI_MSA_PIC_SHIP_MARS:
	case IDI_MSA_PIC_SHIP_URANUS:

		// draw blinking ship lights

		int iColor;

		for (int i = 0; i < 12; i++) {
			iColor = game.nFrame + i;
			if (iColor > 15) iColor -= 15;

			objLight[1] = iColor;
			objLight[4] += 7;
			
			AGI_DrawPic(0, 0, IDF_AGI_PIC_V2 | IDF_AGI_CIRCLE, (Uint8 *)objLight);
		}

		game.nFrame--;
		if (game.nFrame < 0) game.nFrame = 15;

		Mickey_PlaySound(IDI_MSA_SND_PRESS_BLUE);

		break;

	case IDI_MSA_PIC_SHIP_CONTROLS:

		// draw XL30 screen

		if (game.fAnimXL30) {
			if (game.nFrame > 5) game.nFrame = 0;
			Mickey_DrawObj((ENUM_MSA_OBJECT)(IDI_MSA_OBJECT_XL31 + game.nFrame), 0, 4);
			game.nFrame++;
		};

		break;

	default:

		// draw crystal

		if (game.iRoom == IDI_MSA_XTAL_ROOM_XY[game.iPlanet][0]) {
			if (!game.fHasXtal) {
				switch(game.iPlanet) {
				case IDI_MSA_PLANET_VENUS:
					if (game.iRmMenu[game.iRoom] != 2) break;
				default:
					Mickey_DrawObj(
						IDI_MSA_OBJECT_CRYSTAL, 
						IDI_MSA_XTAL_ROOM_XY[game.iPlanet][1], 
						IDI_MSA_XTAL_ROOM_XY[game.iPlanet][2]
					);
					break;
				}
			}
		}

		break;
	}
}

static void Mickey_DrawRoom()
{
	Mickey_DrawRoomPicture();
	Mickey_DrawRoomObjects();
	Mickey_DrawRoomAnimation();
}

static void Mickey_DrawLogo()
{
	char szFile[256] = {0};
	Uint8 *buffer = new Uint8[16384];

	// read in logos.bcg
	sprintf(szFile, IDS_MSA_PATH_LOGO, GetGamePath());
	ReadFile(szFile, buffer);
	
	// draw logo bitmap
	DrawPictureBCG(buffer);
	UpdateScreen();

	delete [] buffer;
}

static void Mickey_Animate()
{
	if ((int)SDL_GetTicks() > (game.nTicks + IDI_MSA_ANIM_DELAY)) {
		game.nTicks = SDL_GetTicks();
		Mickey_DrawRoomAnimation();
	}
}

static void Mickey_PrintRoomDesc()
{
	// print room description

	Mickey_PrintDesc(game.iRoom);
	Mickey_WaitAnyKeyAnim();

	// print extended room description

	if (game.fRmTxt[game.iRoom]) {
		Mickey_PrintExeMsg(game.oRmTxt[game.iRoom] + IDI_MSA_OFS_EXE);
	}
}

static bool Mickey_LoadGame()
{
	FILE *infile;
	char szFile[256] = {0};
	bool diskerror = true;
	int sel;

	while (diskerror) {
		sel = Mickey_GetSel1to9(IDO_MSA_LOAD_GAME[1]);
		if (!sel) return false;

		// load game
		sprintf(szFile, IDS_MSA_PATH_SAVE, GetGamePath(), sel);
		infile = fopen(szFile, "rb");
		if (infile == NULL) {
			Mickey_PrintExeStr(IDO_MSA_CHECK_DISK_DRIVE);
			if (!Mickey_WaitAnyKey()) return false;
		} else {
			if (!fread(&game, 1, sizeof(MSA_GAME), infile)) {
				Mickey_PrintExeStr(IDO_MSA_CHECK_DISK_DRIVE);
				if (!Mickey_WaitAnyKey()) return false;
			} else {
				diskerror = false;
			}
			fclose(infile);			
		}
	}

	Mickey_PrintExeMsg(IDO_MSA_LOAD_GAME[2]);
	return true;
}

static void Mickey_SaveGame()
{
	FILE *outfile;
	char szFile[256] = {0};
	bool diskerror = true;
	int sel;

	bool fOldDisk = Mickey_GetSelYN(IDO_MSA_SAVE_GAME[0], false);

	if (fOldDisk)
		Mickey_PrintExeStr(IDO_MSA_SAVE_GAME[1]);
	else
		Mickey_PrintExeStr(IDO_MSA_SAVE_GAME[2]);

	if (!Mickey_WaitAnyKey()) return;
	
	while (diskerror) {
		sel = Mickey_GetSel1to9(IDO_MSA_SAVE_GAME[3]);
		if (!sel) return;

		if (fOldDisk)
			Mickey_PrintExeStr(IDO_MSA_SAVE_GAME[5]);
		else
			Mickey_PrintExeStr(IDO_MSA_SAVE_GAME[4]);

		if (!Mickey_WaitAnyKey()) return;

		// save game
		sprintf(szFile, IDS_MSA_PATH_SAVE, GetGamePath(), sel);
		outfile = fopen(szFile, "wb");
		if (outfile == NULL) {
			Mickey_PrintExeStr(IDO_MSA_CHECK_DISK_DRIVE);
			if (!Mickey_WaitAnyKey()) return;
		} else {
			if (!fwrite(&game, 1, sizeof(MSA_GAME), outfile)) {
				Mickey_PrintExeStr(IDO_MSA_CHECK_DISK_DRIVE);
				if (!Mickey_WaitAnyKey()) return;
			} else {
				diskerror = false;
			}
			fclose(outfile);			
		}
	}

	Mickey_PrintExeMsg(IDO_MSA_SAVE_GAME[6]);
}

static void Mickey_ShowPlanetInfo()
{
	for (int i = 0; i < IDI_MSA_MAX_PLANET_INFO; i++) {
		Mickey_PrintExeStr(IDO_MSA_PLANET_INFO[game.iPlanet][i]);
		WaitAnyKey();
	}
}

static void Mickey_PrintStory()
{
	char buffer[IDI_MSA_LEN_STORY] = {0};
	char szLine[41] = {0};
	int iRow;
	int pBuf = 0;
	
	Mickey_ReadExe(IDO_MSA_GAME_STORY, (Uint8 *)buffer, sizeof(buffer));
	
	ClearScreen(IDA_DEFAULT);
	for (iRow = 0; iRow < 25; iRow++) {
		strcpy(szLine, buffer + pBuf);
		DrawStr(iRow, 0, szLine);
		pBuf += strlen(szLine) + 1;
	}
	UpdateScreen();
	WaitAnyKey();

	ClearScreen(IDA_DEFAULT);
	for (iRow = 0; iRow < 21; iRow++) {
		strcpy(szLine, buffer + pBuf);
		DrawStr(iRow, 0, szLine);
		pBuf += strlen(szLine) + 1;
	}
	UpdateScreen();
	WaitAnyKey();

	Mickey_DrawRoom();
	UpdateScreen();

	game.fStoryShown = true;
}

static void Mickey_Hidden()
{
	if (game.iRoom == IDI_MSA_PIC_MERCURY_CAVE_0) {
		for (int i = 0; i < 5; i++) {
			Mickey_PrintExeMsg(IDO_MSA_HIDDEN_MSG[i]);
		}
		ClearTextArea();
		UpdateScreen();
		WaitAnyKey();
	}
}

static int Mickey_GetPlanet()
{
	if (!game.nButtons) return -1;

	for (int iPlanet = 0; iPlanet < IDI_MSA_MAX_DAT; iPlanet++) {
		if (!strcmp(IDS_MSA_ADDR_PLANET[iPlanet], game.szAddr)) {
			return iPlanet;
		}
	}

	return -1;
}

static void Mickey_PressOB(int iButton)
{
	char szButtons[12] = {0};

	// check if too many buttons pressed
	if (game.nButtons == IDI_MSA_MAX_BUTTON) {
		game.nButtons = 0;
		memset(game.szAddr, 0, sizeof(game.szAddr));
		Mickey_PrintExeMsg(IDO_MSA_TOO_MANY_BUTTONS_PRESSED);
		return;
	}
	
	// add button press to address
	game.nButtons++;
	game.szAddr[game.nButtons - 1] = (char)iButton;

	// format buttons string
	for (int i = 0; i < IDI_MSA_MAX_BUTTON; i++) {
		szButtons[i * 2] = game.szAddr[i];
		if (game.szAddr[i + 1]) szButtons[(i * 2) + 1] = ',';
	}

	// print pressed buttons
	Mickey_PrintExeStr(IDO_MSA_MICKEY_HAS_PRESSED);
	DrawStr(IDI_MSA_ROW_BUTTONS, IDI_MSA_COL_BUTTONS, szButtons);
	UpdateScreen();
	WaitAnyKey();
}

static void Mickey_CheckAirSupply(bool fSuit, int *iSupply)
{
	if (fSuit) {
		*iSupply -= 1;
		for (int i = 0; i < 4; i++) {
			if (*iSupply == IDI_MSA_AIR_SUPPLY[i]) {
				Mickey_PlaySound(IDI_MSA_SND_XL30);
				Mickey_PrintExeMsg(IDO_MSA_XL30_SPEAKING);
				Mickey_PrintExeMsg(IDO_MSA_AIR_SUPPLY[i]);
				if (i == 3) {
					exit(0);
				}
			}
		}
	} else {
		*iSupply = IDI_MSA_MAX_AIR_SUPPLY;
	}
}

static void Mickey_InsertDisk(int iDisk)
{
	ClearTextArea();
	DrawStr(IDI_MSA_ROW_INSERT_DISK, IDI_MSA_COL_INSERT_DISK, (char *)IDS_MSA_INSERT_DISK[iDisk]);
	UpdateScreen();
	WaitAnyKey();
}

static void Mickey_GameOver()
{
	Mickey_DrawPic(IDI_MSA_PIC_EARTH_SHIP_LEAVING);
	Mickey_PrintExeMsg(IDO_MSA_GAME_OVER[3]);
	Mickey_PlaySound(IDI_MSA_SND_GAME_OVER);

	if (game.fItemUsed[IDI_MSA_ITEM_LETTER]) {
		Mickey_DrawPic(IDI_MSA_PIC_EARTH_MINNIE);
		Mickey_PrintExeMsg(IDO_MSA_GAME_OVER[4]);
		Mickey_PrintExeMsg(IDO_MSA_GAME_OVER[5]);
	} else {
		Mickey_PrintExeMsg(IDO_MSA_GAME_OVER[6]);
		Mickey_PrintExeMsg(IDO_MSA_GAME_OVER[7]);
	}

	WaitAnyKey();
	exit(0);
}

static void Mickey_FlipSwitch()
{
	if (game.fHasXtal || game.nXtals) {
		if (!game.fStoryShown) {
			Mickey_PrintStory();
			Mickey_Randomize();
		}

		// activate screen animation
		game.fAnimXL30 = true;

		ClearTextArea();
		UpdateScreen();
		Mickey_PlaySound(IDI_MSA_SND_XL30);
		Mickey_PrintExeMsg(IDO_MSA_XL30_SPEAKING);

		if (game.fHasXtal) {
			game.fHasXtal = false;
			Mickey_PrintExeMsg(IDO_MSA_CRYSTAL_PIECE_FOUND);
		}
		
		if (game.nXtals == IDI_MSA_MAX_PLANET) {
			Mickey_PrintExeMsg(IDO_MSA_GAME_OVER[0]);
			Mickey_PrintExeMsg(IDO_MSA_GAME_OVER[1]);
			Mickey_PrintExeMsg(IDO_MSA_GAME_OVER[2]);

#ifdef _DEBUG
			strcpy(game.szAddr, (char *)IDS_MSA_ADDR_PLANET[IDI_MSA_PLANET_EARTH]);
			game.nButtons = strlen(game.szAddr);
#endif

		} else {
			Mickey_PrintExeStr(game.iClue[game.nXtals]);

#ifdef _DEBUG
			DrawStr(24, 12, (char *)IDS_MSA_NAME_PLANET_2[game.iPlanetXtal[game.nXtals]]);
			DrawStr(24, 22, (char *)IDS_MSA_ADDR_PLANET[game.iPlanetXtal[game.nXtals]]);
			strcpy(game.szAddr, (char *)IDS_MSA_ADDR_PLANET[game.iPlanetXtal[game.nXtals]]);
			game.nButtons = strlen(game.szAddr);
			UpdateScreen();
#endif

			Mickey_WaitAnyKeyAnim();
		}
	} else {
		Mickey_PrintStory();
	}
}

static void Mickey_Inventory()
{
	int iRow = IDI_MSA_ROW_INV_ITEMS;
	char szCrystals[12] = {0};

	sprintf(szCrystals, IDS_MSA_CRYSTALS, IDS_MSA_CRYSTAL_NO[game.nXtals]);

	ClearScreen(IDA_DEFAULT);
	DrawStr(IDI_MSA_ROW_INV_TITLE, IDI_MSA_COL_INV_TITLE, IDS_MSA_INVENTORY);
	DrawStr(IDI_MSA_ROW_INV_CRYSTALS, IDI_MSA_COL_INV_ITEMS, szCrystals);
	
	for (int iItem = 0; iItem < IDI_MSA_MAX_ITEM; iItem++) {
		if (game.fItem[game.iItem[iItem]] && (game.iItem[iItem] != IDI_MSA_OBJECT_NONE)) {
			DrawStr(iRow++, IDI_MSA_COL_INV_ITEMS, (char *)IDS_MSA_NAME_ITEM[game.iItem[iItem]]);
		}
	}

	UpdateScreen();
	WaitAnyKey();

	ClearScreen(IDA_DEFAULT);
}

static void Mickey_Randomize()
{
	int iPlanet;
	int iHint;
	bool done;

	memset(game.iPlanetXtal, 0, sizeof(game.iPlanetXtal));
	memset(game.iClue, 0, sizeof(game.iClue));

	game.iPlanetXtal[0] = IDI_MSA_PLANET_EARTH;
	game.iPlanetXtal[8] = IDI_MSA_PLANET_URANUS;

	for (int i = 1; i < 9; i++) {
		if (i == 8) {
			iPlanet = IDI_MSA_PLANET_URANUS;
		} else {
			done = false;
			while (!done) {
				iPlanet = Rnd(IDI_MSA_MAX_PLANET);
				done = true;
				for (int j = 0; j < IDI_MSA_MAX_PLANET; j++) {
					if (game.iPlanetXtal[j] == iPlanet) {
						done = false;
						break;
					}
				}
			}
		}

		game.iPlanetXtal[i] = iPlanet;
		
		done = false;
		while (!done) {
			iHint = Rnd(5);
			done = true;
		}

		game.iClue[i] = IDO_MSA_NEXT_PIECE[iPlanet][iHint];
	}
}

static void Mickey_FlashScreen()
{
	Mickey_PlaySound(IDI_MSA_SND_PRESS_BLUE);

	ClearGfxScreen(15);
	UpdateScreen();

	Mickey_DrawRoom();
	Mickey_PrintDesc(game.iRoom);
	UpdateScreen();
}

static void Mickey_Intro()
{
	// draw sierra logo
	Mickey_DrawLogo();

	// draw title picture
	game.iRoom = IDI_MSA_PIC_TITLE;
	Mickey_DrawRoom();
	
	// show copyright and play theme
	Mickey_PrintExeMsg(IDO_MSA_COPYRIGHT);
	Mickey_PlaySound(IDI_MSA_SND_THEME);
	
	// load game
	game.fIntro = true;
	if (Mickey_GetSelYN(IDO_MSA_LOAD_GAME[0], true)) {
		if (Mickey_LoadGame()) {
			game.iPlanet = IDI_MSA_PLANET_EARTH;
			game.fIntro = false;
			game.iRoom = IDI_MSA_PIC_SHIP_CORRIDOR;
			return;
		}
	}

	// show spaceship landing scene
	game.iPlanet = IDI_MSA_PLANET_EARTH;
	game.iRoom = IDI_MSA_PIC_EARTH_ROAD_4;

	Mickey_DrawRoom();
	Mickey_PrintRoomDesc();

#ifndef _DEBUG
	Mickey_PlaySound(IDI_MSA_SND_SHIP_LAND);
#endif

	Mickey_FlashScreen();
	Mickey_FlashScreen();
	Mickey_FlashScreen();

	Mickey_PrintExeMsg(IDO_MSA_INTRO);
}

static void Mickey_GetItem(ENUM_MSA_ITEM iItem)
{
	game.fItem[iItem] = true;
	game.iItem[game.nItems++] = iItem;
	game.fRmTxt[game.iRoom] = 0;
	Mickey_PlaySound(IDI_MSA_SND_TAKE);
	Mickey_DrawRoom();
}

static void Mickey_GetXtal(int iStr)
{
	game.fRmTxt[game.iRoom] = 0;
	game.fHasXtal = true;
	game.nXtals++;
	Mickey_PlaySound(IDI_MSA_SND_CRYSTAL);
	Mickey_DrawRoom();
	Mickey_PrintDatMessage(iStr);
}

static bool Mickey_Parse(int cmd, int arg)
{
	switch(cmd) {

	// BASIC

	case IDI_MSA_ACTION_GOTO_ROOM:
		game.iRoom = arg;
		return true;
	case IDI_MSA_ACTION_SHOW_INT_STR:
		Mickey_PrintExeMsg(IDO_MSA_ERROR[arg]);
		break;
	case IDI_MSA_ACTION_SHOW_DAT_STR:
		Mickey_PrintDatMessage(arg);
		break;

	// GENERAL

	case IDI_MSA_ACTION_PLANET_INFO:
		Mickey_ShowPlanetInfo();
		break;
	case IDI_MSA_ACTION_SAVE_GAME:
		Mickey_SaveGame();
		break;
	case IDI_MSA_ACTION_LOOK_MICKEY:
		Mickey_PrintExeMsg(IDO_MSA_YOU_CAN_SEE_MICKEY_ALREADY);
		break;

	// EARTH

	case IDI_MSA_ACTION_GET_ROPE:
		if (game.iRmMenu[game.iRoom] == 2) {
			game.iRmObj[game.iRoom] = IDI_MSA_OBJECT_NONE;
			game.iRmMenu[game.iRoom] = 3;
			Mickey_GetItem(IDI_MSA_ITEM_ROPE);
			Mickey_PrintExeMsg(IDO_MSA_ERROR[7]);
		} else {
			game.iRmMenu[game.iRoom] = 1;
			Mickey_PrintDatMessage(11);
		}
		break;
	case IDI_MSA_ACTION_UNTIE_ROPE:
		game.iRmPic[game.iRoom] = IDI_MSA_PIC_EARTH_TIRE_SWING_1;
		game.iRmObj[game.iRoom] = 0;
		game.iRmMenu[game.iRoom] = 2;
		Mickey_DrawRoom();
		Mickey_PrintDatMessage(12);
		break;
	case IDI_MSA_ACTION_GET_BONE:
		game.iRmObj[game.iRoom] = IDI_MSA_OBJECT_NONE;
		game.iRmMenu[game.iRoom] = 1;
		Mickey_GetItem(IDI_MSA_ITEM_BONE);
		Mickey_PrintDatMessage(arg);
		break;
	case IDI_MSA_ACTION_GET_XTAL_EARTH:
		game.iRmMenu[game.iRoom] = 1;
		Mickey_GetXtal(arg);
		break;
	case IDI_MSA_ACTION_LOOK_DESK:
		game.iRmMenu[game.iRoom] = 1;
		game.iRmObj[game.iRoom] = 2;
		Mickey_DrawRoom();
		Mickey_PrintDatMessage(arg);
		break;
	case IDI_MSA_ACTION_WRITE_LETTER:
		game.iRmMenu[game.iRoom] = 3;
		game.iRmMenu[IDI_MSA_PIC_EARTH_MAILBOX] = 1;
		game.iRmObj[game.iRoom] = IDI_MSA_OBJECT_NONE;
		Mickey_GetItem(IDI_MSA_ITEM_LETTER);
		Mickey_PrintDatMessage(arg);
		break;
	case IDI_MSA_ACTION_MAIL_LETTER:
		game.fItemUsed[IDI_MSA_ITEM_LETTER] = true;
		game.fItem[IDI_MSA_ITEM_LETTER] = false;
		game.iRmMenu[game.iRoom] = 0;
		Mickey_PrintDatMessage(arg);
		break;
	case IDI_MSA_ACTION_OPEN_MAILBOX:
		if (game.fItemUsed[IDI_MSA_ITEM_LETTER]) {
			Mickey_PrintDatMessage(110);
		} else {
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_OPEN_CUPBOARD:
		if (game.iRmMenu[game.iRoom]) {
			if (game.iRmObj[game.iRoom] == IDI_MSA_OBJECT_NONE) {
				Mickey_PrintDatMessage(78);
			} else {
				Mickey_PrintDatMessage(arg);
			}
		} else {
			game.iRmMenu[game.iRoom] = 1;
			game.iRmPic[game.iRoom] = IDI_MSA_PIC_EARTH_KITCHEN_1;
			game.iRmObj[game.iRoom] = 3;
			Mickey_DrawRoom();
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_GET_FLASHLIGHT:
		if (game.fItem[IDI_MSA_ITEM_FLASHLIGHT]) {
			Mickey_PrintDatMessage(90);
		} else {
			game.iRmObj[game.iRoom] = IDI_MSA_OBJECT_NONE;
			Mickey_GetItem(IDI_MSA_ITEM_FLASHLIGHT);
			Mickey_DrawRoom();
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_OPEN_CABINET:
		if (game.iRmMenu[game.iRoom]) {
			Mickey_PrintDatMessage(109);
		} else {
			game.iRmMenu[game.iRoom] = 1;
			game.iRmPic[game.iRoom] = IDI_MSA_PIC_EARTH_GARAGE_1;
			game.iRmObj[game.iRoom] = 15;
			Mickey_DrawRoom();
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_GET_CROWBAR:
		if (game.fItem[IDI_MSA_ITEM_CROWBAR]) {
			Mickey_PrintDatMessage(90);
		} else {
			game.iRmObj[game.iRoom]--;
			Mickey_GetItem(IDI_MSA_ITEM_CROWBAR);
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_GET_WRENCH:
		if (game.fItem[IDI_MSA_ITEM_WRENCH]) {
			Mickey_PrintDatMessage(90);
		} else {
			game.iRmObj[game.iRoom] -= 2;
			Mickey_GetItem(IDI_MSA_ITEM_WRENCH);
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_OPEN_CLOSET:
		if (game.iRmMenu[game.iRoom]) {
			Mickey_PrintDatMessage(99);
		} else {
			game.iRmMenu[game.iRoom] = 1;
			game.iRmPic[game.iRoom] = IDI_MSA_PIC_EARTH_BEDROOM_1;
			game.iRmObj[game.iRoom] = 7;
			Mickey_DrawRoom();
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_GET_MATTRESS:
		if (game.fItem[IDI_MSA_ITEM_MATTRESS]) {
			Mickey_PrintDatMessage(90);
		} else {
			game.iRmObj[game.iRoom]--;
			Mickey_GetItem(IDI_MSA_ITEM_MATTRESS);
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_GET_SCARF:
		if (game.fItem[IDI_MSA_ITEM_SCARF]) {
			Mickey_PrintDatMessage(90);
		} else {
			game.iRmObj[game.iRoom] -= 2;
			Mickey_GetItem(IDI_MSA_ITEM_SCARF);
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_GET_SUNGLASSES:
		if (game.fItem[IDI_MSA_ITEM_SUNGLASSES]) {
			Mickey_PrintDatMessage(90);
		} else {
			game.iRmObj[game.iRoom]--;
			Mickey_GetItem(IDI_MSA_ITEM_SUNGLASSES);
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_GET_SCALE:
		if (game.fItem[IDI_MSA_ITEM_SCALE]) {
			Mickey_PrintDatMessage(90);
		} else {
			game.iRmMenu[IDI_MSA_PIC_VENUS_WEIGH] = 1;
			game.iRmMenu[IDI_MSA_PIC_NEPTUNE_WEIGH] = 1;
			game.iRmMenu[IDI_MSA_PIC_MERCURY_WEIGH] = 1;
			game.iRmMenu[IDI_MSA_PIC_SATURN_WEIGH] = 1;
			game.iRmMenu[IDI_MSA_PIC_PLUTO_WEIGH] = 1;
			game.iRmMenu[IDI_MSA_PIC_JUPITER_WEIGH] = 1;
			game.iRmMenu[IDI_MSA_PIC_MARS_WEIGH] = 1;
			game.iRmMenu[IDI_MSA_PIC_URANUS_WEIGH] = 1;
			game.iRmObj[game.iRoom] -= 2;
			Mickey_GetItem(IDI_MSA_ITEM_SCALE);
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_GOTO_SPACESHIP:
		game.iRoom = IDI_MSA_PIC_SHIP_AIRLOCK;
		if (game.iPlanet != IDI_MSA_PLANET_EARTH)
			Mickey_InsertDisk(0);
		return true;

	// VENUS

	case IDI_MSA_ACTION_DOWN_CHASM:
		if (game.fItem[IDI_MSA_ITEM_ROPE]) {
			game.iRmMenu[game.iRoom] = 1;
		}
		Mickey_PrintDatMessage(arg);
		break;
	case IDI_MSA_ACTION_DOWN_ROPE:
		if (game.fItemUsed[IDI_MSA_ITEM_ROPE]) {
			game.iRoom = IDI_MSA_PIC_VENUS_PROBE;
			return true;
		} else {
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_USE_ROPE:
		if (game.fItemUsed[IDI_MSA_ITEM_ROPE]) {
			Mickey_PrintDatMessage(22);
		} else {
			game.fItemUsed[IDI_MSA_ITEM_ROPE] = true;
			game.fItem[IDI_MSA_ITEM_ROPE] = false;
			game.iRmPic[game.iRoom] = IDI_MSA_PIC_VENUS_CHASM_1;
			Mickey_DrawRoom();
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_OPEN_HATCH:
		if (game.fItemUsed[IDI_MSA_ITEM_WRENCH]) {
			if ((game.iRmMenu[game.iRoom] == 3) || (game.iRmPic[game.iRoom] == IDI_MSA_PIC_VENUS_PROBE_1))
				Mickey_PrintDatMessage(39);
			else {
				game.iRmMenu[game.iRoom] = 2;
				game.iRmPic[game.iRoom] = IDI_MSA_PIC_VENUS_PROBE_1;
				Mickey_DrawRoom();
				Mickey_PrintDatMessage(24);
			}
		} else {
			if (game.fItem[IDI_MSA_ITEM_WRENCH]) {
				game.iRmMenu[game.iRoom] = 1;
			}
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_USE_WRENCH:
		game.fItemUsed[IDI_MSA_ITEM_WRENCH] = true;
		Mickey_PrintDatString(arg);
		if (game.iRmPic[game.iRoom] == IDI_MSA_PIC_VENUS_PROBE_1) {
			ClearRow(22);
		}
		UpdateScreen();
		WaitAnyKey();
		break;
	case IDI_MSA_ACTION_GET_XTAL_VENUS:
		game.iRmMenu[game.iRoom] = 3;
		Mickey_GetXtal(arg);
		break;

	// TRITON (NEPTUNE)

	case IDI_MSA_ACTION_LOOK_CASTLE:
		if (!game.iRmMenu[game.iRoom]) {
			game.iRmMenu[game.iRoom] = 1;
		}
		Mickey_PrintDatMessage(arg);
		break;
	case IDI_MSA_ACTION_ENTER_OPENING:
		if (game.fItemUsed[IDI_MSA_ITEM_CROWBAR]) {
			game.iRoom = IDI_MSA_PIC_NEPTUNE_CASTLE_4;
			return true;
		} else {
			if (game.fItem[IDI_MSA_ITEM_CROWBAR]) {
				game.iRmMenu[game.iRoom] = 2;
			}
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_USE_CROWBAR:
		game.fItemUsed[IDI_MSA_ITEM_CROWBAR] = true;
		game.iRmMenu[game.iRoom] = 1;
		game.iRmPic[game.iRoom] = IDI_MSA_PIC_NEPTUNE_ENTRANCE_1;
		Mickey_DrawRoom();
		Mickey_PrintDatMessage(arg);
		break;
	case IDI_MSA_ACTION_GET_XTAL_NEPTUNE:
		if (game.fHasXtal) {
			Mickey_PrintDatMessage(71);
		} else {
			if (game.fItem[IDI_MSA_ITEM_SCARF]) {
				game.iRmMenu[game.iRoom] = 1;
			}
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_TALK_LEADER:
		game.iRoom = IDI_MSA_PIC_NEPTUNE_ENTRYWAY;
		Mickey_PrintDatMessage(arg);
		return true;
	case IDI_MSA_ACTION_GIVE_SCARF:
		game.iRmObj[game.iRoom] = 18;
		Mickey_GetXtal(arg);
		game.fItem[IDI_MSA_ITEM_SCARF] = false;
		game.iRmMenu[game.iRoom] = 0;
		game.iRmMenu[IDI_MSA_PIC_EARTH_BEDROOM] = 2;
		game.iRoom = IDI_MSA_PIC_NEPTUNE_ENTRYWAY;
		return true;

	// MERCURY

	case IDI_MSA_ACTION_GET_XTAL_MERCURY:
		if (game.fHasXtal) {
			game.iRmMenu[game.iRoom] = 2;
			Mickey_PrintDatMessage(32);
		} else {
			if (game.fItem[IDI_MSA_ITEM_SUNGLASSES]) {
				game.iRmMenu[game.iRoom] = 1;
			}
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_GIVE_SUNGLASSES:
		game.iRmObj[game.iRoom] = 17;
		game.iRmMenu[game.iRoom] = 2;
		game.fItem[IDI_MSA_ITEM_SUNGLASSES] = false;
		Mickey_GetXtal(arg);
		break;

	// TITAN (SATURN)

	case IDI_MSA_ACTION_CROSS_LAKE:
		if (game.fItem[IDI_MSA_ITEM_MATTRESS]) {
			game.iRmMenu[IDI_MSA_PIC_SATURN_LAKE_0] = 1;
			game.iRmMenu[IDI_MSA_PIC_SATURN_LAKE_1] = 1;
			game.iRmMenu[IDI_MSA_PIC_SATURN_LAKE_2] = 1;
		}
		Mickey_PrintDatMessage(arg);
		break;
	case IDI_MSA_ACTION_USE_MATTRESS:
		game.iRoom = IDI_MSA_PIC_SATURN_ISLAND;
		Mickey_PrintDatMessage(arg);
		return true;
	case IDI_MSA_ACTION_GET_XTAL_SATURN:
		if (game.fHasXtal) {
			Mickey_PrintDatMessage(29);
		} else {
			Mickey_GetXtal(arg);
		}
		break;
	case IDI_MSA_ACTION_LEAVE_ISLAND:
		game.iRoom = IDI_MSA_PIC_SATURN_LAKE_1;
		Mickey_PrintDatMessage(arg);
		return true;

	// PLUTO

	case IDI_MSA_ACTION_GET_XTAL_PLUTO:
		if (game.fHasXtal) {
			Mickey_PrintDatMessage(19);
		} else {
			if (game.fItem[IDI_MSA_ITEM_BONE]) {
				game.iRmMenu[game.iRoom] = 1;
			}
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_GIVE_BONE:
		game.fItem[IDI_MSA_ITEM_BONE] = false;
		game.iRmMenu[game.iRoom] = 0;
		game.iRmObj[game.iRoom] = 16;
		Mickey_GetXtal(arg);
		break;

	// IO (JUPITER)

	case IDI_MSA_ACTION_GET_ROCK_0:
		if (game.fItem[IDI_MSA_ITEM_ROCK]) {
			Mickey_PrintDatMessage(38);
		} else {
			game.iRmMenu[game.iRoom] = 1;
			game.iRmObj[game.iRoom] = IDI_MSA_OBJECT_NONE;
			Mickey_GetItem(IDI_MSA_ITEM_ROCK);
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_GET_ROCK_1:
		if (game.fItem[IDI_MSA_ITEM_ROCK]) {
			Mickey_PrintDatMessage(38);
		} else {
			game.iRmMenu[game.iRoom] = 1;
			game.iRmObj[game.iRoom] = IDI_MSA_OBJECT_NONE;
			Mickey_GetItem(IDI_MSA_ITEM_ROCK);
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_GET_XTAL_JUPITER:
		if (game.fHasXtal) {
			Mickey_PrintDatMessage(15);
		} else {
			switch (game.nRocks) {
			case 0:
				if (game.fItem[IDI_MSA_ITEM_ROCK]) {
					game.iRmMenu[game.iRoom] = 1;
				}
				Mickey_PrintDatMessage(arg);
				break;
			case 1:
				if (game.fItem[IDI_MSA_ITEM_ROCK]) {
					game.iRmMenu[game.iRoom] = 1;
				}
				Mickey_PrintDatMessage(34);
				break;
			case 2:
				Mickey_GetXtal(35);
				break;
			}
		}
		break;
	case IDI_MSA_ACTION_THROW_ROCK:
		game.fItem[IDI_MSA_ITEM_ROCK] = false;
		game.nItems--;
		game.iRmObj[game.iRoom]++;
		game.iRmMenu[game.iRoom] = 0;
		Mickey_DrawRoom();
		if (game.nRocks) {
			Mickey_PrintDatMessage(37);
		} else {
			Mickey_PrintDatMessage(arg);
		}
		game.nRocks++;
		break;

	// MARS

	case IDI_MSA_ACTION_GO_TUBE:
		if (game.fItem[IDI_MSA_ITEM_FLASHLIGHT]) {
			game.iRmMenu[game.iRoom] = 1;
		}
		Mickey_PrintDatMessage(arg);
		break;
	case IDI_MSA_ACTION_USE_FLASHLIGHT:
		game.iRoom = IDI_MSA_PIC_MARS_TUBE_1;
		Mickey_PrintDatMessage(15);
		return true;
	case IDI_MSA_ACTION_PLUTO_DIG:
		if (game.fHasXtal) {
			Mickey_PrintDatMessage(21);
		} else {
			Mickey_GetXtal(arg);
		}
		break;
	case IDI_MSA_ACTION_GET_XTAL_MARS:
		if (game.fHasXtal) {
			Mickey_PrintDatMessage(23);
		} else {
			Mickey_PrintDatMessage(arg);
		}
		break;

	// OBERON (URANUS)

	case IDI_MSA_ACTION_ENTER_TEMPLE:
		game.iRoom = IDI_MSA_PIC_URANUS_TEMPLE;
		return true;
	case IDI_MSA_ACTION_USE_CRYSTAL:
		if (game.iRmMenu[game.iRoom]) {
			Mickey_PrintDatMessage(25);
		} else {
			game.iRmMenu[game.iRoom] = 1;
			game.iRmPic[game.iRoom] = IDI_MSA_PIC_URANUS_TEMPLE_1;
			Mickey_DrawRoom();
			game.iRmPic[game.iRoom] = IDI_MSA_PIC_URANUS_TEMPLE;
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_OPEN_DOOR:
		if (game.fTempleDoorOpen) {
			Mickey_PrintDatMessage(36);
		} else {
			game.fTempleDoorOpen = 1;
			game.iRmPic[game.iRoom] = IDI_MSA_PIC_URANUS_TEMPLE_2;
			Mickey_DrawRoom();
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_ENTER_DOOR:
		if (game.fTempleDoorOpen) {
			game.iRoom = IDI_MSA_PIC_URANUS_STEPS;
			return true;
		} else {
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_GET_XTAL_URANUS:
		if (game.fHasXtal) {
			Mickey_PrintDatMessage(34);
		} else {
			if (game.fItem[IDI_MSA_ITEM_CROWBAR]) {
				game.iRmMenu[game.iRoom] = 1;
			}
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_USE_CROWBAR_1:
		game.iRmMenu[game.iRoom] = 0;
		Mickey_GetXtal(arg);
		break;

	// SPACESHIP

	case IDI_MSA_ACTION_GO_NORTH:
		if (game.fShipDoorOpen) {
			if (game.fSuit) {
				Mickey_PrintDatMessage(45);
			} else {
				game.iRoom = IDI_MSA_PIC_SHIP_CORRIDOR;
				return true;
			}
		} else {
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_GO_PLANET:
		if (!game.fShipDoorOpen) {
			if ((game.nXtals == IDI_MSA_MAX_PLANET) && (game.iPlanet == IDI_MSA_PLANET_EARTH))
				Mickey_GameOver();
			if ((game.iPlanet == game.iPlanetXtal[game.nXtals]) || (game.iPlanet == IDI_MSA_PLANET_EARTH)) {
				game.fHasXtal = false;
				game.iRoom = IDI_MSA_HOME_PLANET[game.iPlanet];
				if (game.iPlanet != IDI_MSA_PLANET_EARTH)
					Mickey_InsertDisk(1);
				return true;
			} else {
				game.iRoom = IDI_MSA_SHIP_PLANET[game.iPlanet];
				return true;
			}
		} else {
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_PRESS_BUTTON:
		if (game.fShipDoorOpen) {		// inner door open
			if (game.iPlanet && !game.fSuit) {
				Mickey_PrintDatMessage(arg);
			} else {
				game.fShipDoorOpen = false;
				game.iRmPic[game.iRoom]--;
				Mickey_DrawRoom();
				Mickey_PrintDatMessage(2);
			}
		} else {
			game.fShipDoorOpen = true;
			game.iRmPic[game.iRoom]++;
			Mickey_DrawRoom();
			Mickey_PrintDatMessage(14);
		}
		break;
	case IDI_MSA_ACTION_WEAR_SPACESUIT:
		if (game.fSuit) {
			if (game.fShipDoorOpen) {
				game.fSuit = false;
				game.iRmMenu[game.iRoom] = 0;
				game.iRmPic[game.iRoom] -= 2;
				Mickey_DrawRoom();
				Mickey_PrintDatMessage(13);
			} else {
				Mickey_PrintDatMessage(3);
			}
		} else {
			if (game.iPlanet) {
				game.fSuit = true;
				game.iRmMenu[game.iRoom] = 1;
				game.iRmPic[game.iRoom] += 2;
				Mickey_DrawRoom();
				Mickey_PrintDatMessage(arg);
			} else {
				Mickey_PrintDatMessage(12);
			}
		}
		break;
	case IDI_MSA_ACTION_READ_GAUGE:
		Mickey_PrintDatString(arg);
		DrawStr(IDI_MSA_ROW_TEMPERATURE, IDI_MSA_COL_TEMPERATURE_C, 
			(char *)IDS_MSA_TEMP_C[game.iPlanet]);
		DrawStr(IDI_MSA_ROW_TEMPERATURE, IDI_MSA_COL_TEMPERATURE_F, 
			(char *)IDS_MSA_TEMP_F[game.iPlanet]);
		UpdateScreen();
		WaitAnyKey();
		break;
	case IDI_MSA_ACTION_PRESS_ORANGE:
		if (game.fFlying) {
			Mickey_PrintDatMessage(4);
		} else {
			Mickey_PlaySound(IDI_MSA_SND_PRESS_ORANGE);
			Mickey_PrintDatMessage(arg);
			Mickey_PressOB(IDI_MSA_BUTTON_ORANGE);
		}
		break;
	case IDI_MSA_ACTION_PRESS_BLUE:
		if (game.fFlying) {
			Mickey_PrintDatMessage(4);
		} else {
			Mickey_PlaySound(IDI_MSA_SND_PRESS_BLUE);
			Mickey_PrintDatMessage(arg);
			Mickey_PressOB(IDI_MSA_BUTTON_BLUE);
		}
		break;
	case IDI_MSA_ACTION_FLIP_SWITCH:
		Mickey_FlipSwitch();
		break;
	case IDI_MSA_ACTION_PUSH_THROTTLE:
		if (game.fFlying) {
			game.fFlying = false;
			game.nButtons = 0;
			memset(game.szAddr, 0, sizeof(game.szAddr));
			Mickey_DrawRoom();
			Mickey_PrintDatString(22);
			DrawStr(IDI_MSA_ROW_PLANET, IDI_MSA_COL_PLANET, 
				(char *)IDS_MSA_PLANETS[game.iPlanet]);
			UpdateScreen();
			Mickey_WaitAnyKeyAnim();
			Mickey_ShowPlanetInfo();
		} else {
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_PULL_THROTTLE:
		if (game.fFlying) {
			Mickey_PrintDatMessage(18);
		} else {
			if (Mickey_GetPlanet() != -1) {
				game.fFlying = true;
				game.iPlanet = Mickey_GetPlanet();
				Mickey_DrawRoom();
				Mickey_PrintDatMessage(16);
			} else {
				game.nButtons = 0;
				memset(game.szAddr, 0, sizeof(game.szAddr));
				Mickey_PrintDatMessage(17);
			}
		}
		break;
	case IDI_MSA_ACTION_LEAVE_ROOM:
		if (game.fFlying) {
			Mickey_PrintDatMessage(24);
		} else {
			game.iRoom = arg;
			return true;
		}
		break;
	case IDI_MSA_ACTION_OPEN_CABINET_1:
		if (game.iRmMenu[game.iRoom]) {
			Mickey_PrintExeMsg(IDO_MSA_THE_CABINET_IS_ALREADY_OPEN);
		} else {
			game.iRmMenu[game.iRoom] = 1;
			game.iRmPic[game.iRoom] = IDI_MSA_PIC_SHIP_KITCHEN_1;
			Mickey_DrawRoom();
			Mickey_PrintDatMessage(arg);
		}
		break;
	case IDI_MSA_ACTION_READ_MAP:
		game.iRmPic[game.iRoom] = IDI_MSA_PIC_STAR_MAP;
		Mickey_DrawRoom();
		Mickey_PrintDatMessage(46);
		Mickey_PrintDatMessage(47);
		Mickey_PrintDatMessage(48);
		game.iRmPic[game.iRoom] = IDI_MSA_PIC_SHIP_BEDROOM;
		Mickey_DrawRoom();
		break;
	case IDI_MSA_ACTION_GO_WEST:
		game.nButtons = 0;
		memset(game.szAddr, 0, sizeof(game.szAddr));
		game.iRoom = arg;
		return true;
	}

	return false;
}

static void Mickey_GameLoop()
{
	char *buffer = new char[sizeof(MSA_MENU)];
	MSA_MENU menu;
	int iSel0, iSel1;
	bool done;

	for (;;) {
		Mickey_DrawRoom();

		if (game.fIntro) {
			game.fIntro = false;
		} else {
			Mickey_PrintRoomDesc();
		}
		
		if (game.iRoom == IDI_MSA_PIC_NEPTUNE_GUARD) {
			game.iRoom = IDI_MSA_PIC_NEPTUNE_LEADER;
			done = true;
		} else {
			done = false;
		}

		while (!done) {
			Mickey_CheckAirSupply(game.fSuit, &game.nAir);
			Mickey_ReadMenu(game.iRoom, buffer);
			memcpy(&menu, buffer, sizeof(MSA_MENU));
			Mickey_PatchMenu(&menu);
			memcpy(buffer, &menu, sizeof(MSA_MENU));
			Mickey_GetMenuSel(buffer, &iSel0, &iSel1);
			done = Mickey_Parse(menu.cmd[iSel0].data[iSel1], menu.arg[iSel0].data[iSel1]);
		}

		game.nFrame = 0;
	}

	delete [] buffer;
}

// Debug

static void Debug_Mickey_DrawObjs()
{
	char szTitle[14] = {0};

	for (int iObj = 0; iObj < IDI_MSA_MAX_OBJ; iObj++) {
		Mickey_DrawPic(0);
		Mickey_DrawObj((ENUM_MSA_OBJECT)iObj, 0, 0);

		ClearTextArea();
		sprintf(szTitle, "Object %d", iObj);
		DrawStrAttrMiddle(22, IDA_DEFAULT, szTitle);
		DrawStrAttrMiddle(23, IDA_DEFAULT, (char *)IDS_MSA_NAME_OBJ[iObj]);
		UpdateScreen();
		WaitAnyKey();
	}
}

static void Debug_Mickey_DrawPics()
{
	char szTitle[14] = {0};

	for (int iPic = 1; iPic <= IDI_MSA_MAX_PIC; iPic++) {
		Mickey_DrawPic(iPic);

		ClearTextArea();
		sprintf(szTitle, "Picture %d", iPic);
		DrawStrAttrMiddle(22, IDA_DEFAULT, szTitle);
		UpdateScreen();
		WaitAnyKey();
	}
}

// Init

static void Mickey_InitVars()
{
	Uint8 buffer[512];

	// clear game struct
	memset(&game, 0, sizeof(game));
	memset(&game.iItem, IDI_MSA_OBJECT_NONE, sizeof(game.iItem));

	// read room extended desc flags
	Mickey_ReadExe(IDO_MSA_ROOM_TEXT, buffer, sizeof(buffer));
	memcpy(game.fRmTxt, buffer, sizeof(game.fRmTxt));

	// read room extended desc offsets
	Mickey_ReadExe(IDO_MSA_ROOM_TEXT_OFFSETS, buffer, sizeof(buffer));
	memcpy(game.oRmTxt, buffer, sizeof(game.oRmTxt));

	// read room object indices
	Mickey_ReadExe(IDO_MSA_ROOM_OBJECT, buffer, sizeof(buffer));
	memcpy(game.iRmObj, buffer, sizeof(game.iRmObj));
	
	// read room picture indices
	//Mickey_ReadExe(IDO_MSA_ROOM_PICTURE, buffer, sizeof(buffer));
	//memcpy(game.iRmPic, buffer, sizeof(game.iRmPic));

	// read room menu patch indices
	Mickey_ReadExe(IDO_MSA_ROOM_MENU_FIX, buffer, sizeof(buffer));
	memcpy(game.nRmMenu, buffer, sizeof(game.nRmMenu));

	// set room picture indices
	for (int i = 0; i < IDI_MSA_MAX_ROOM; i++) {
		game.iRmPic[i] = i;
	}
	game.iRmPic[IDI_MSA_PIC_SHIP_AIRLOCK] = IDI_MSA_PIC_SHIP_AIRLOCK_0;

#ifdef _DEBUG

	game.iPlanet = IDI_MSA_PLANET_EARTH;
	game.iRoom = IDI_MSA_PIC_SHIP_CONTROLS;
	game.fHasXtal = true;
	game.nXtals = 9;
	game.fItemUsed[IDI_MSA_ITEM_LETTER] = true;

#endif

}

static void Mickey_InitEngine()
{
	SetScreenPar(320, 200, (char*)mickey_fontdata);
}

void Mickey_Init()
{
	Mickey_InitEngine();
	Mickey_InitVars();
	Mickey_Intro();
	Mickey_GameLoop();
	Mickey_GameOver();
}

// Driver

GAME (mickey, 1986, "The Walt Disney Company", "Mickey's Space Adventure", 0, Mickey_Init)
