/*
 *  Copyright (C) 2003-2004 Curt Coder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <string>
#include <SDL.h>
#include "agipic.h"
#include "driver.h"
#include "fileio.h"
#include "font.h"
#include "gfx.h"
#include "selgame.h"
#include "ui.h"
#include "troll.h"

static int var_TRO_room = 1;
static int var_TRO_moves = 0;
static int var_TRO_missing = IDI_TRO_MAX_TREASURE;

// User Interface

static void Troll_WaitAnyKey()
{
	DrawStr(24, 4, IDS_TRO_PRESSANYKEY);
	UpdateScreen();
	WaitAnyKey();
}

static void Troll_GetMenuSel(char *szMenu, int *iSel, int nSel)
{
	SDL_Event event;
	int x, y;
	
	DrawMenu(szMenu, *iSel);

	for (;;) {
		while (SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_QUIT:
				exit(0);
			case SDL_MOUSEMOTION:
				x = event.motion.x / GetFontWidth();
				y = event.motion.y / GetFontHeight();
				GetMouseMenuSel(iSel, nSel, x, y);
				DrawMenu(szMenu, *iSel);
				break;
			case SDL_MOUSEBUTTONUP:
				switch(event.button.button) {
				case SDL_BUTTON_LEFT:
					return;
				}
				break;
			case SDL_KEYDOWN:
				switch (event.key.keysym.sym) {
				case SDLK_t:
				case SDLK_f:
					Troll_Inventory();
					*iSel = 0;
					DrawMenu(szMenu, *iSel);
					break;
				case SDLK_SPACE:
					*iSel += 1;
					if (*iSel == nSel) *iSel = IDI_TRO_SEL_OPTION_1;
					DrawMenu(szMenu, *iSel);
					break;
				case SDLK_RETURN:
				case SDLK_KP_ENTER:
					return;
				}
				break;
			}
		}
	}
}

// Graphics

static void Troll_DrawPic(int iPic, bool fDrawObj)
{
	Uint8 *buffer = (Uint8 *)malloc(4096);
	char szFile[256] = {0};

	// construct filename
	sprintf(szFile, IDS_TRO_PATH_PIC, GetGamePath(), iPic);

	// read picture file into buffer
	ReadFile(szFile, buffer);

	// draw picture
	ClearScreenAGI(IDI_TRO_PIC_WIDTH, IDI_TRO_PIC_HEIGHT, IDI_TRO_PIC_X0, IDI_TRO_PIC_Y0, IDI_TRO_PIC_FLAGS);
	AGI_DrawPic(IDI_TRO_PIC_X0, IDI_TRO_PIC_Y0, IDI_TRO_PIC_FLAGS, buffer);
	UpdateScreen();

	free(buffer);
}

// Game Logic

static void Troll_Inventory()
{
	char szMissing[40];

	ClearScreen(0x07);
	DrawStr(1, 12, IDS_TRO_TREASURE_0);
	DrawStr(2, 12, IDS_TRO_TREASURE_1);

	switch (var_TRO_missing) {
	case 1:
		sprintf(szMissing, IDS_TRO_TREASURE_5, var_TRO_missing);
		DrawStr(20, 10, szMissing);
		break;
	case 0:
		DrawStr(20, 1, IDS_TRO_TREASURE_6);
		break;
	case IDI_TRO_MAX_TREASURE:
		DrawStr(3, 17, IDS_TRO_TREASURE_2);
	default:
		sprintf(szMissing, IDS_TRO_TREASURE_4, var_TRO_missing);
		DrawStr(20, 10, szMissing);
		break;
	}	

	DrawStr(24, 6, IDS_TRO_PRESSANYKEY);
	UpdateScreen();
	WaitAnyKey();
}

static void Troll_WaitAnyKeyIntro()
{
	SDL_Event event;
	int iMsg = 0;

	for (;;) {
		while (SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_QUIT:
				exit(0);
			case SDL_MOUSEBUTTONUP:
				switch(event.button.button) {
				case SDL_BUTTON_LEFT:
					return;
				}
				break;
			case SDL_KEYDOWN:
				return;
			}
		}
		if (!iMsg) {
			DrawStr(22, 3, IDS_TRO_INTRO_2);
			UpdateScreen();
			iMsg = 1;
		} else {
			DrawStr(22, 3, IDS_TRO_INTRO_3);
			UpdateScreen();
			iMsg = 0;
		}
		Delay(0x28);
	}
}

static void Troll_Credits()
{
	ClearScreen(0x07);

	DrawStr(1, 2, IDS_TRO_CREDITS_0);

	DrawStrAttr(7, 19, 10, "T");
	DrawStrAttr(7, 20, 11, "R");
	DrawStrAttr(7, 21, 12, "O");
	DrawStrAttr(7, 22, 13, "L");
	DrawStrAttr(7, 23, 14, "L");
	DrawStrAttr(7, 24, 15, "'");
	DrawStrAttr(7, 25, 9, "S");
	DrawStrAttr(7, 27, 11, "T");
	DrawStrAttr(7, 28, 12, "A");
	DrawStrAttr(7, 29, 13, "L");
	DrawStrAttr(7, 30, 14, "E");
	DrawStrAttr(7, 32, 9, "(");
	DrawStrAttr(7, 33, 10, "t");
	DrawStrAttr(7, 34, 11, "m");
	DrawStrAttr(7, 35, 12, ")");

	DrawStr(8, 19, IDS_TRO_CREDITS_2);

	DrawStrAttr(13, 11, 9, IDS_TRO_CREDITS_3);
	DrawStrAttr(15, 8, 10, IDS_TRO_CREDITS_4);
	DrawStrAttr(17, 7, 12, IDS_TRO_CREDITS_5);
	DrawStrAttr(19, 2, 14, IDS_TRO_CREDITS_6);
	
	UpdateScreen();

	Troll_WaitAnyKey();
}

static void Troll_Tutorial()
{
	bool done = false;
	int iSel = 0;
	char szTreasure[16] = {0};

	for (;;) {
		//SetGfxMode();
		PrintStr(IDS_TRO_TUTORIAL_0);
		WaitForKey(SDLK_SPACE);
		
		done = false;
		while (!done) {
			Troll_GetMenuSel(IDS_TRO_TUTORIAL_1, &iSel, IDI_TRO_MAX_OPTION);
			switch(iSel) {
			case IDI_TRO_SEL_OPTION_1:
				ClearGfxScreen(0x0B);
				UpdateScreen();
				break;
			case IDI_TRO_SEL_OPTION_2:
				ClearGfxScreen(0x00);
				UpdateScreen();
				break;
			case IDI_TRO_SEL_OPTION_3:
				done = true;
				break;
			}
		}

		// do you need more practice ?
		ClearScreen(0x4F);
		DrawStr(7, 4, IDS_TRO_TUTORIAL_5);
		DrawStr(9, 4, IDS_TRO_TUTORIAL_6);
		UpdateScreen();
		if (!GetSelYN()) break;
	}

	// show info texts
	ClearScreen(0x5F);
	DrawStr(4, 1, IDS_TRO_TUTORIAL_7);
	DrawStr(5, 1, IDS_TRO_TUTORIAL_8);
	UpdateScreen();
	Troll_WaitAnyKey();

	ClearScreen(0x2F);
	DrawStr(6, 1, IDS_TRO_TUTORIAL_9);
	UpdateScreen();
	Troll_WaitAnyKey();
	
	ClearScreen(0x19);
	DrawStr(7, 1, IDS_TRO_TUTORIAL_10);
	DrawStr(8, 1, IDS_TRO_TUTORIAL_11);
	UpdateScreen();
	Troll_WaitAnyKey();
	
	ClearScreen(0x6E);
	DrawStr(9, 1, IDS_TRO_TUTORIAL_12);
	DrawStr(10, 1, IDS_TRO_TUTORIAL_13);
	UpdateScreen();
	Troll_WaitAnyKey();
	
	ClearScreen(0x4C);
	DrawStr(11, 1, IDS_TRO_TUTORIAL_14);
	DrawStr(12, 1, IDS_TRO_TUTORIAL_15);
	UpdateScreen();
	Troll_WaitAnyKey();
	
	ClearScreen(0x5D);
	DrawStr(13, 1, IDS_TRO_TUTORIAL_16);
	DrawStr(14, 1, IDS_TRO_TUTORIAL_17);
	DrawStr(15, 1, IDS_TRO_TUTORIAL_18);
	UpdateScreen();
	Troll_WaitAnyKey();

	// show treasures
	ClearScreen(0x2A);
	DrawStr(2, 1, IDS_TRO_TUTORIAL_19);
	for (int i = 0; i < IDI_TRO_MAX_TREASURE; i++)
		DrawStr(19 - i, 11, (char *)IDS_TRO_NAME_TREASURE[i]);
	UpdateScreen();
	Troll_WaitAnyKey();
}

static void Troll_Intro()
{
	// sierra on-line presents
	ClearScreen(0x2F);
	DrawStr(9, 10, IDS_TRO_INTRO_0);
	DrawStr(14, 15, IDS_TRO_INTRO_1);
	UpdateScreen();

	// draw troll picture
	Delay(0x32);
	ClearScreen(0x0F);
	//////DrawPic(0x5A);
	//Troll_DrawPic(47, false);
	UpdateScreen();
	
	// wait for keypress and alternate message
	Troll_WaitAnyKeyIntro();	

	// have you played this game before?
	DrawStr(22, 3, IDS_TRO_INTRO_4);
	DrawStr(23, 6, IDS_TRO_INTRO_5);
	UpdateScreen();
	
	if (!GetSelYN()) 
		Troll_Tutorial();

	Troll_Credits();
}

static void Troll_GameOver()
{
	char szMoves[40];

	ClearScreen(0x0f); // hack

	ClearTextArea();
	//DrawPic(0);

	ClearTextArea();
	//DrawPic(0);

	sprintf(szMoves, IDS_TRO_GAMEOVER_0, var_TRO_moves);
	DrawStr(21, 1, szMoves);
	DrawStr(22, 1, IDS_TRO_GAMEOVER_1);
	UpdateScreen();
	Troll_WaitAnyKey();
}

static void Troll_GameLoop()
{
	int iSel;
	//int nSel;
	bool done = false;
	//char szMsg[161];
	//char szMenu[161];
	
	while (!done) {
//		Troll_DrawRoomPic(var_TRO_room);
//		Troll_ReadRoomText(var_TRO_room, szMsg, szMenu, &nSel);

		iSel = 0;
		//Troll_GetMenuSel(szMsg, szMenu, &iSel, nSel);
		var_TRO_moves++;

		switch(iSel) {
		case IDI_TRO_SEL_OPTION_1:
			break;
		case IDI_TRO_SEL_OPTION_2:
			break;
		case IDI_TRO_SEL_OPTION_3:
			break;
		}
	}

}

// Init

static void Troll_InitEngine()
{
	SetScreenPar(320, 200, (char*)ibm_fontdata);
	SetMenuPars(20, 21, " *");
}

void Troll_Init()
{
	Troll_InitEngine();
	Troll_Intro();
	//Troll_GameLoop();
	Troll_GameOver();
}

// Driver

GAME (troll, 1984, "Sierra On-Line", "Troll's Tale", GAME_NOT_WORKING | GAME_NO_SOUND, Troll_Init)
