/*
 *  Copyright (C) 2003-2004 Curt Coder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <string>
#include <SDL.h>
#include "agipic.h"
#include "driver.h"
#include "fileio.h"
#include "font.h"
#include "gfx.h"
#include "selgame.h"
#include "ui.h"
#include "winnie.h"

static WTP_SAVE_GAME	game;
static int				room;
static int				mist = -1;
static bool				wind = false;
static bool				event = false;
static int				tigger_mist;

static int Winnie_GetObjInRoom(int iRoom);

// File Operations

static void Winnie_ReadRoom(int iRoom, Uint8 *buffer, int buflen)
{
	char szFile[256] = {0};
	sprintf(szFile, IDS_WTP_PATH_ROOM, GetGamePath(), iRoom);
	ReadFile(szFile, buffer);
}

static void Winnie_ReadObj(int iObj, Uint8 *buffer, int buflen)
{
	char szFile[256] = {0};
	sprintf(szFile, IDS_WTP_PATH_OBJ, GetGamePath(), iObj);
	ReadFile(szFile, buffer);
}

static void Winnie_ReadSaveGame(Uint8 *buffer)
{
	char szFile[256] = {0};
	sprintf(szFile, IDS_WTP_PATH, GetGamePath(), IDS_WTP_FILE_SAVEGAME);
	ReadFile(szFile, buffer);
}

static void Winnie_WriteSaveGame(Uint8 *buffer)
{
	char szFile[256] = {0};
	sprintf(szFile, IDS_WTP_PATH, GetGamePath(), IDS_WTP_FILE_SAVEGAME);
	WriteFile(szFile, buffer, sizeof(WTP_SAVE_GAME));
}

// User Interface

static void Winnie_PrintRoomStr(int iRoom, int iStr)
{
	WTP_ROOM_HDR hdr;
	Uint8 *buffer = (Uint8 *)malloc(4096);

	Winnie_ReadRoom(iRoom, buffer, 4096);
	memcpy(&hdr, buffer, sizeof(hdr));
	PrintStrXOR((char *)(buffer + hdr.ofsStr[iStr - 1] - IDI_WTP_OFS_ROOM));

	free(buffer);
}

static void Winnie_PrintObjStr(int iObj, int iStr)
{
	WTP_OBJ_HDR hdr;
	Uint8 *buffer = (Uint8 *)malloc(2048);

	Winnie_ReadObj(iObj, buffer, 2048);
	memcpy(&hdr, buffer, sizeof(hdr));
	PrintStrXOR((char *)(buffer + hdr.ofsStr[iStr] - IDI_WTP_OFS_OBJ));

	free(buffer);
}

static void Winnie_GetMenuMouseSel(int *iSel, int fCanSel[], int x, int y)
{
	switch(y) {
	case IDI_WTP_ROW_OPTION_1:
		if (fCanSel[IDI_WTP_SEL_OPT_1])	*iSel = IDI_WTP_SEL_OPT_1;
		break;
	case IDI_WTP_ROW_OPTION_2:
		if (fCanSel[IDI_WTP_SEL_OPT_2])	*iSel = IDI_WTP_SEL_OPT_2;
		break;
	case IDI_WTP_ROW_OPTION_3:
		if (fCanSel[IDI_WTP_SEL_OPT_3])	*iSel = IDI_WTP_SEL_OPT_3;
		break;
	case IDI_WTP_ROW_OPTION_4:
		if (fCanSel[IDI_WTP_SEL_NORTH] && (x > IDI_WTP_COL_NORTH - 1) && (x < 6)) *iSel = IDI_WTP_SEL_NORTH;
		if (fCanSel[IDI_WTP_SEL_SOUTH] && (x > IDI_WTP_COL_SOUTH - 1) && (x < 13)) *iSel = IDI_WTP_SEL_SOUTH;
		if (fCanSel[IDI_WTP_SEL_EAST] && (x > IDI_WTP_COL_EAST - 1) && (x < 19)) *iSel = IDI_WTP_SEL_EAST;
		if (fCanSel[IDI_WTP_SEL_WEST] && (x > IDI_WTP_COL_WEST - 1) && (x < 25)) *iSel = IDI_WTP_SEL_WEST;
		if (fCanSel[IDI_WTP_SEL_TAKE] && (x > IDI_WTP_COL_TAKE - 1) && (x < 33)) *iSel = IDI_WTP_SEL_TAKE;
		if (fCanSel[IDI_WTP_SEL_DROP] && (x > IDI_WTP_COL_DROP - 1) && (x < 39)) *iSel = IDI_WTP_SEL_DROP;
		break;
	}
}

static void Winnie_DrawMenu(char *szMenu, int iSel, int fCanSel[])
{
	int iRow, iCol;

	ClearTextArea();
	DrawStr(IDI_WTP_ROW_MENU, IDI_WTP_COL_MENU, szMenu);

	if (fCanSel[IDI_WTP_SEL_NORTH])
		DrawStr(IDI_WTP_ROW_OPTION_4, IDI_WTP_COL_NSEW, IDS_WTP_NSEW);
	if (fCanSel[IDI_WTP_SEL_TAKE])
		DrawStr(IDI_WTP_ROW_OPTION_4, IDI_WTP_COL_TAKE, IDS_WTP_TAKE);
	if (fCanSel[IDI_WTP_SEL_DROP])
		DrawStr(IDI_WTP_ROW_OPTION_4, IDI_WTP_COL_DROP, IDS_WTP_DROP);
	
	switch(iSel) {
	case IDI_WTP_SEL_OPT_1:
		iRow = IDI_WTP_ROW_OPTION_1;
		iCol = IDI_WTP_COL_OPTION;
		break;
	case IDI_WTP_SEL_OPT_2:
		iRow = IDI_WTP_ROW_OPTION_2;
		iCol = IDI_WTP_COL_OPTION;
		break;
	case IDI_WTP_SEL_OPT_3:
		iRow = IDI_WTP_ROW_OPTION_3;
		iCol = IDI_WTP_COL_OPTION;
		break;
	case IDI_WTP_SEL_NORTH:
		iRow = IDI_WTP_ROW_OPTION_4;
		iCol = IDI_WTP_COL_NORTH;
		break;
	case IDI_WTP_SEL_SOUTH:
		iRow = IDI_WTP_ROW_OPTION_4;
		iCol = IDI_WTP_COL_SOUTH;
		break;
	case IDI_WTP_SEL_EAST:
		iRow = IDI_WTP_ROW_OPTION_4;
		iCol = IDI_WTP_COL_EAST;
		break;
	case IDI_WTP_SEL_WEST:
		iRow = IDI_WTP_ROW_OPTION_4;
		iCol = IDI_WTP_COL_WEST;
		break;
	case IDI_WTP_SEL_TAKE:
		iRow = IDI_WTP_ROW_OPTION_4;
		iCol = IDI_WTP_COL_TAKE;
		break;
	case IDI_WTP_SEL_DROP:
		iRow = IDI_WTP_ROW_OPTION_4;
		iCol = IDI_WTP_COL_DROP;
		break;
	}
	DrawStr(iRow, iCol - 1, IDS_WTP_SELECTION);
	UpdateScreen();
}

static void Winnie_ClrMenuSel(int *iSel, int fCanSel[])
{
	*iSel = IDI_WTP_SEL_OPT_1;
	while(!fCanSel[*iSel]) {
		*iSel += 1;
	}
}

static void Winnie_IncMenuSel(int *iSel, int fCanSel[])
{
	do {
		*iSel += 1;
		if (*iSel > IDI_WTP_SEL_DROP) *iSel = IDI_WTP_SEL_OPT_1;
	} while(!fCanSel[*iSel]);
}

static void Winnie_DecMenuSel(int *iSel, int fCanSel[])
{
	do {
		*iSel -= 1;
		if (*iSel < IDI_WTP_SEL_OPT_1) *iSel = IDI_WTP_SEL_DROP;
	} while(!fCanSel[*iSel]);
}

#define Winnie_MakeSel() {\
	if (fCanSel[*iSel]) {\
		return;\
	} else {\
		Winnie_KeyHelp();\
		Winnie_ClrMenuSel(iSel, fCanSel);\
	}\
}

static void Winnie_GetMenuSel(char *szMenu, int *iSel, int fCanSel[])
{
	SDL_Event event;
	int x, y;

	Winnie_ClrMenuSel(iSel, fCanSel);
	Winnie_DrawMenu(szMenu, *iSel, fCanSel);

	for (;;) {
		// check if tigger/mist is to be triggered
//			if (something)
//				event = true;

		while (SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_QUIT:
				exit(0);
			case SDL_MOUSEMOTION:
				x = event.motion.x / GetFontWidth();
				y = event.motion.y / GetFontHeight();
				Winnie_GetMenuMouseSel(iSel, fCanSel, x, y);
				break;
			case SDL_MOUSEBUTTONUP:
				switch(event.button.button) {
				case SDL_BUTTON_LEFT:
					event.type = SDL_KEYDOWN;
					event.key.keysym.sym = SDLK_RETURN;
					SDL_PushEvent(&event);
					break;
				case SDL_BUTTON_RIGHT:
					*iSel = IDI_WTP_SEL_BACK;
					return;
				case SDL_BUTTON_MIDDLE:
					Winnie_Inventory();
					break;
				case SDL_BUTTON_WHEELUP:
					event.type = SDL_KEYDOWN;
					event.key.keysym.sym = SDLK_UP;
					SDL_PushEvent(&event);
					break;
				case SDL_BUTTON_WHEELDOWN:
					event.type = SDL_KEYDOWN;
					event.key.keysym.sym = SDLK_DOWN;
					SDL_PushEvent(&event);
					break;
				}
				break;
			case SDL_KEYDOWN:
				switch (event.key.keysym.sym) {
				case SDLK_F11:
					FlipCGA();
					Winnie_DrawRoomPic();
					break;
				case SDLK_F12:
					FlipDblSize();
					Winnie_DrawRoomPic();
					break;
				case SDLK_LCTRL:
				case SDLK_RCTRL:
					break;
				case SDLK_ESCAPE:
					*iSel = IDI_WTP_SEL_HOME;
					return;
				case SDLK_BACKSPACE:
					*iSel = IDI_WTP_SEL_BACK;
					return;
				case SDLK_c:
					Winnie_Inventory();
					break;
				case SDLK_SPACE:
				case SDLK_RIGHT:
				case SDLK_DOWN:
					Winnie_IncMenuSel(iSel, fCanSel);
					break;
				case SDLK_LEFT:
				case SDLK_UP:
					Winnie_DecMenuSel(iSel, fCanSel);
					break;
				case SDLK_1:
				case SDLK_2:
				case SDLK_3:
					*iSel = event.key.keysym.sym - SDLK_1;
					if (fCanSel[*iSel + IDI_WTP_SEL_REAL_OPT_1]) {
						return;
					} else {
						Winnie_KeyHelp();
						Winnie_ClrMenuSel(iSel, fCanSel);
					}
					break;
				case SDLK_n:
					*iSel = IDI_WTP_SEL_NORTH;
					Winnie_MakeSel();
					break;
				case SDLK_s:
					if (event.key.keysym.mod & KMOD_CTRL) {
						FlipSound();
					} else {
						*iSel = IDI_WTP_SEL_SOUTH;
						Winnie_MakeSel();
					}
					break;
				case SDLK_e:
					*iSel = IDI_WTP_SEL_EAST;
					Winnie_MakeSel();
					break;
				case SDLK_w:
					*iSel = IDI_WTP_SEL_WEST;
					Winnie_MakeSel();
					break;
				case SDLK_t:
					*iSel = IDI_WTP_SEL_TAKE;
					Winnie_MakeSel();
					break;
				case SDLK_d:
					*iSel = IDI_WTP_SEL_DROP;
					Winnie_MakeSel();
					break;
				case SDLK_RETURN:
				case SDLK_KP_ENTER:
					switch(*iSel) {
					case IDI_WTP_SEL_OPT_1:
					case IDI_WTP_SEL_OPT_2:
					case IDI_WTP_SEL_OPT_3:
						for (int iSel2 = 0; iSel2 < IDI_WTP_MAX_OPTION; iSel2++) {
							if (*iSel == (fCanSel[iSel2 + IDI_WTP_SEL_REAL_OPT_1] - 1)) {
								*iSel = iSel2;
								return;
							}
						}
						break;
					default:
						if (fCanSel[*iSel]) {
							return;
						}
						break;
					}
				default:
					Winnie_KeyHelp();
					Winnie_ClrMenuSel(iSel, fCanSel);
					break;
				}
				break;
			}

			Winnie_DrawMenu(szMenu, *iSel, fCanSel);
		}
	}
}

static bool Winnie_GetSelOkBack()
{
	SDL_Event event;

	for (;;) {
		while (SDL_PollEvent(&event)) {
			if (event.type == SDL_QUIT) 
				exit(0);
			if (event.type == SDL_MOUSEBUTTONUP) {
				switch(event.button.button) {
				case SDL_BUTTON_LEFT:
					return true;
				case SDL_BUTTON_RIGHT:
					return false;
				}
			}
			if (event.type == SDL_KEYDOWN) {
				switch (event.key.keysym.sym) {
				case SDLK_BACKSPACE:
					return false;
				default:
					return true;
				}
			}
		}
	}
}

// Sound

static bool Winnie_PlaySound(ENUM_WTP_SOUND iSound)
{
	char szFile[256] = {0};
	Uint8 *buffer = new Uint8[2048];
	bool fOk;
	delete [] buffer;

return true; //

	// read file into buffer
	sprintf(szFile, IDS_WTP_PATH_SND, GetGamePath(), iSound);
	ReadFile(szFile, buffer);

	// play sound
	
	delete [] buffer;

	return fOk;
}

static void Winnie_Inventory()
{
	char szMissing[41] = {0};

	if (game.iObjHave)
		Winnie_PrintObjStr(game.iObjHave, IDI_WTP_OBJ_TAKE);
	else {
		ClearTextArea();
		DrawStr(IDI_WTP_ROW_MENU, IDI_WTP_COL_MENU, IDS_WTP_INVENTORY_0);
	}

	sprintf(szMissing, IDS_WTP_INVENTORY_1, game.nObjMiss);
	DrawStr(IDI_WTP_ROW_OPTION_4, IDI_WTP_COL_MENU, szMissing);
	UpdateScreen();
	WaitAnyKey();
}

static void Winnie_KeyHelp()
{
	Winnie_PlaySound(IDI_WTP_SND_KEYHELP);
	PrintMsg(IDS_WTP_HELP_0);
	PrintMsg(IDS_WTP_HELP_1);
}

// Graphics

static void Winnie_DrawPic(char *szName)
{
	char szFile[256] = {0};
	Uint8 *buffer = new Uint8[4096];

	// construct filename
	sprintf(szFile, IDS_WTP_PATH, GetGamePath(), szName);

	// read picture file into buffer
	ReadFile(szFile, buffer);

	// draw picture
	ClearScreenAGI(IDI_WTP_PIC_WIDTH, IDI_WTP_PIC_HEIGHT, 
		IDI_WTP_PIC_X0, IDI_WTP_PIC_Y0, IDI_WTP_PIC_FLAGS);
	AGI_DrawPic(IDI_WTP_PIC_X0, IDI_WTP_PIC_Y0, IDI_WTP_PIC_FLAGS, buffer);

	delete [] buffer;
}

static void Winnie_DrawObjPic(int iObj, int x0, int y0)
{
	WTP_OBJ_HDR	objhdr;
	Uint8 *buffer = new Uint8[2048];

	if (!iObj) return;

	Winnie_ReadObj(iObj, buffer, 2048);
	memcpy(&objhdr, buffer, sizeof(WTP_OBJ_HDR));

	AGI_DrawPic(x0, y0, IDI_WTP_PIC_FLAGS, buffer + objhdr.ofsPic - IDI_WTP_OFS_OBJ);

	delete [] buffer;
}

static void Winnie_DrawRoomPic()
{
	WTP_ROOM_HDR roomhdr;
	Uint8 *buffer = new Uint8[4096];
	int iObj = Winnie_GetObjInRoom(room);

	// clear gfx screen
	ClearScreenAGI(IDI_WTP_PIC_WIDTH, IDI_WTP_PIC_HEIGHT, 
		IDI_WTP_PIC_X0, IDI_WTP_PIC_Y0, IDI_WTP_PIC_FLAGS);

	// read room picture
	Winnie_ReadRoom(room, buffer, 4096);
	memcpy(&roomhdr, buffer, sizeof(WTP_ROOM_HDR));

	// draw room picture
	AGI_DrawPic(IDI_WTP_PIC_X0, IDI_WTP_PIC_Y0, IDI_WTP_PIC_FLAGS, buffer + roomhdr.ofsPic - IDI_WTP_OFS_ROOM);

	// draw object picture
	Winnie_DrawObjPic(iObj, IDI_WTP_PIC_X0 + roomhdr.objX, IDI_WTP_PIC_Y0 + roomhdr.objY);

	delete [] buffer;
}

// Game Logic

static void Winnie_GameOver()
{
	// sing the Pooh song forever
	for (;;) {
		PrintStr(IDS_WTP_SONG_0);
		Winnie_PlaySound(IDI_WTP_SND_POOH_0);
		PrintStr(IDS_WTP_SONG_1);
		Winnie_PlaySound(IDI_WTP_SND_POOH_1);
		PrintStr(IDS_WTP_SONG_2);
		Winnie_PlaySound(IDI_WTP_SND_POOH_2);
		WaitAnyKey();
	}
}

static void Winnie_SaveGame()
{
	Uint8 *buffer = new Uint8[sizeof(WTP_SAVE_GAME)];
	memcpy(buffer, &game, sizeof(WTP_SAVE_GAME));
	Winnie_WriteSaveGame(buffer);
	delete [] buffer;
}

static void Winnie_LoadGame()
{
	Uint8 *buffer = new Uint8[sizeof(WTP_SAVE_GAME)];
	Winnie_ReadSaveGame(buffer);
	memcpy(&game, buffer, sizeof(WTP_SAVE_GAME));
	delete [] buffer;
}

// Object Handling

static int Winnie_GetObjInRoom(int iRoom)
{
	for (int iObj = 1; iObj < IDI_WTP_MAX_ROOM_OBJ; iObj++) 
		if (game.iObjRoom[iObj] == iRoom) 
			return iObj;
	return 0;
}

static bool Winnie_IsRightObj(int iRoom, int iObj, int *iCode)
{
	WTP_ROOM_HDR roomhdr;
	WTP_OBJ_HDR	objhdr;
	Uint8 *roomdata = new Uint8[4096];
	Uint8 *objdata = new Uint8[2048];

	Winnie_ReadRoom(iRoom, roomdata, 4096);
	memcpy(&roomhdr, roomdata, sizeof(WTP_ROOM_HDR));
	Winnie_ReadObj(iObj, objdata, 2048);
	memcpy(&objhdr, objdata, sizeof(WTP_OBJ_HDR));

	delete [] roomdata;
	delete [] objdata;

	*iCode = objhdr.objId;

	if (objhdr.objId == 11) objhdr.objId = 34;

	if (roomhdr.objId == objhdr.objId)
		return true;
	else
		return false;
}

static void Winnie_TakeObj(int iRoom)
{
	if (game.iObjHave)
		// player is already carrying an object, can't take
		PrintMsg(IDS_WTP_CANT_TAKE);
	else {
		// take object
		int iObj = Winnie_GetObjInRoom(iRoom);
		game.iObjHave = iObj;
		game.iObjRoom[iObj] = 0;

		PrintStr(IDS_WTP_OK);
		Winnie_PlaySound(IDI_WTP_SND_TAKE);

		Winnie_DrawRoomPic();

		// print object "take" string
		Winnie_PrintObjStr(game.iObjHave, IDI_WTP_OBJ_TAKE);
		WaitAnyKey();

		// HACK WARNING
		if (iObj == 18) {
			game.fGame[0x0d] = 1;
		}
	}
}

static void Winnie_DropObj(int iRoom)
{
	int iCode;

	if (Winnie_GetObjInRoom(iRoom)) {
		// there already is an object in the room, can't drop
		PrintMsg(IDS_WTP_CANT_DROP);
	} else {
		// HACK WARNING
		if (game.iObjHave == 18) {
			game.fGame[0x0d] = 0;
		}

		if (Winnie_IsRightObj(iRoom, game.iObjHave, &iCode)) {
			// object has been dropped in the right place
			PrintMsg(IDS_WTP_OK);
			Winnie_PlaySound(IDI_WTP_SND_DROP_OK);
			Winnie_PrintObjStr(game.iObjHave, IDI_WTP_OBJ_DROP);
			WaitAnyKey();

			// increase amount of objects returned, decrease amount of objects missing
			game.nObjMiss--;
			game.nObjRet++;
			
			// xor the dropped object with 0x80 to signify it has been dropped in the right place
			for (int i = 0; i < IDI_WTP_MAX_OBJ_MISSING; i++) {
				if (game.iUsedObj[i] == game.iObjHave) {
					game.iUsedObj[i] ^= 0x80;
					break;
				}
			}

			// set flag according to dropped object's id
			game.fGame[iCode] = 1;
			
			// player is carrying nothing
			game.iObjHave = 0;
			
			if (!game.nObjMiss) {
				// all objects returned, tell player to find party
				//Winnie_PlaySound(IDI_WTP_SND_FANFARE);
				PrintMsg(IDS_WTP_GAME_OVER_0);
				PrintMsg(IDS_WTP_GAME_OVER_1);
			}
		} else {
			// drop object in the given room
			game.iObjRoom[game.iObjHave] = iRoom;

			// object has been dropped in the wrong place
			PrintStr(IDS_WTP_WRONG_PLACE);
			Winnie_PlaySound(IDI_WTP_SND_DROP);
			Winnie_DrawRoomPic();
			PrintMsg(IDS_WTP_WRONG_PLACE);

			// print object description
			Winnie_PrintObjStr(game.iObjHave, IDI_WTP_OBJ_DESC);
			WaitAnyKey();

			game.iObjHave = 0;
		}
	}
}

static void Winnie_DropObjRnd()
{
	if (!game.iObjHave) return;
	
	int iRoom;
	bool done = false;

	while (!done) {
		iRoom = Rnd(IDI_WTP_MAX_ROOM_NORMAL);
		done = true;
		if (iRoom == room)
			done = false;
		for (int j = 0; j < IDI_WTP_MAX_ROOM_OBJ; j++) {
			if (game.iObjRoom[j] == iRoom) {
				done = false;
			}
		}
	}

	game.iObjRoom[game.iObjHave] = iRoom;
	game.iObjHave = 0;
}

// Game Events

static void Winnie_Wind()
{
	int iRoom;
	bool done;

	wind = 0;
	game.nMoves = 0;
	if (!game.nObjMiss) return;

	PrintStr(IDS_WTP_WIND_0);
	Winnie_PlaySound(IDI_WTP_SND_WIND_0);
	WaitAnyKey();
	PrintStr(IDS_WTP_WIND_1);
	Winnie_PlaySound(IDI_WTP_SND_WIND_0);
	WaitAnyKey();

	Winnie_DropObjRnd();

	// randomize positions of objects at large
	for (int i = 0; i < IDI_WTP_MAX_OBJ_MISSING; i++) {
		if (!(game.iUsedObj[i] & IDI_XOR_KEY)) {
			done = false;
			while (!done) {
				iRoom = Rnd(IDI_WTP_MAX_ROOM_NORMAL);
				done = true;
				for (int j = 0; j < IDI_WTP_MAX_ROOM_OBJ; j++) {
					if (game.iObjRoom[j] == iRoom) {
						done = false;
					}
				}
			}
			game.iObjRoom[game.iUsedObj[i]] = iRoom;
		}
	}
}

static void Winnie_Mist()
{
	// mist length in turns is (2-5)
	mist = Rnd(4) + 2;
	room = IDI_WTP_ROOM_MIST;
	Winnie_DrawRoomPic();
	PrintMsg(IDS_WTP_MIST);
}

static void Winnie_Tigger()
{
	room = IDI_WTP_ROOM_TIGGER;
	Winnie_DrawRoomPic();
	PrintMsg(IDS_WTP_TIGGER);
	Winnie_DropObjRnd();
}

// Game Logic

static void Winnie_ShowOwlHelp()
{
	if (game.iObjHave) {
		PrintMsg(IDS_WTP_OWL_0);
		Winnie_PrintObjStr(game.iObjHave, IDI_WTP_OBJ_HELP);
		WaitAnyKey();
	}
	if (Winnie_GetObjInRoom(room)) {
		PrintMsg(IDS_WTP_OWL_0);
		Winnie_PrintObjStr(Winnie_GetObjInRoom(room), IDI_WTP_OBJ_HELP);
		WaitAnyKey();
	}
}

static void Winnie_SetFlag(int iFlag)
{
	game.fGame[iFlag] = 1;
}

static void Winnie_ClearFlag(int iFlag)
{
	game.fGame[iFlag] = 0;
}

#define Winnie_SetTakeDrop() {\
	if (Winnie_GetObjInRoom(room))\
		fCanSel[IDI_WTP_SEL_TAKE] = true;\
	else\
		fCanSel[IDI_WTP_SEL_TAKE] = false;\
	if (game.iObjHave)\
		fCanSel[IDI_WTP_SEL_DROP] = true;\
	else\
		fCanSel[IDI_WTP_SEL_DROP] = false;\
}

static int Winnie_Parser(int pc, int index, Uint8 *buffer)
{
	WTP_ROOM_HDR hdr;
	int startpc = pc;
	Uint8 opcode;
	int iNewRoom = 0;

	int iSel, iDir, iBlock;
	int fCanSel[IDI_WTP_SEL_LAST + 1];
	char szMenu[121] = {0};
	bool done;
	int fBlock;

	// extract header from buffer
	memcpy(&hdr, buffer, sizeof(WTP_ROOM_HDR));

	for (;;) {
		pc = startpc;

		// check if block is to be run

		iBlock = *(buffer + pc++);
		if (iBlock == 0)
			return IDI_WTP_PAR_OK;

		fBlock = *(buffer + pc++);
		if (game.fGame[iBlock] != fBlock)
			return IDI_WTP_PAR_OK;

		// extract text from block

		opcode = *(buffer + pc);
		switch(opcode) {
		case 0:
		case IDO_WTP_OPTION_0:
		case IDO_WTP_OPTION_1:
		case IDO_WTP_OPTION_2:
			// clear fCanSel block
			memset(fCanSel, 0, sizeof(fCanSel));

			// check if NSEW directions should be displayed
			if (hdr.roomNew[0])
				fCanSel[IDI_WTP_SEL_NORTH] = fCanSel[IDI_WTP_SEL_SOUTH] = 
				fCanSel[IDI_WTP_SEL_EAST] = fCanSel[IDI_WTP_SEL_WEST] = true;

			// check if object in room or player carrying one
			Winnie_SetTakeDrop();
			
			// check which rows have a menu option
			for (iSel = 0; iSel < IDI_WTP_MAX_OPTION; iSel++) {
				opcode = *(buffer + pc++);
				if (opcode) {
					fCanSel[opcode - IDO_WTP_OPTION_0] = true;
					fCanSel[iSel + IDI_WTP_SEL_REAL_OPT_1] = opcode - 0x14;
				}
			}

			// extract menu string
			strcpy(szMenu, (char *)(buffer + pc));
			XOR80(szMenu);
			break;
		default:
			// print description
			PrintStrXOR((char *)(buffer + pc));
			if (Winnie_GetSelOkBack())
				return IDI_WTP_PAR_OK;
			else 
				return IDI_WTP_PAR_BACK;
		}

		// input handler

		done = false;
		while (!done) {
			// run wind if it's time
			if (wind)
				Winnie_Wind();

			// get menu selection
			Winnie_GetMenuSel(szMenu, &iSel, fCanSel);

			if (++game.nMoves == IDI_WTP_MAX_MOVES_UNTIL_WIND)
				wind = true;

			if (event && (room <= IDI_WTP_MAX_ROOM_TELEPORT)) {
				if (!tigger_mist) {
					tigger_mist = 1;
					Winnie_Tigger();
				} else {
					tigger_mist = 0;
					Winnie_Mist();
				}
				event = false;
				return IDI_WTP_PAR_GOTO;
			}

			// process selection
			switch(iSel) {
			case IDI_WTP_SEL_HOME:
				switch(room) {
				case IDI_WTP_ROOM_HOME:
				case IDI_WTP_ROOM_MIST:
				case IDI_WTP_ROOM_TIGGER:
					break;
				default:
					room = IDI_WTP_ROOM_HOME;
					return IDI_WTP_PAR_GOTO;
				}
				break;
			case IDI_WTP_SEL_BACK:
				return IDI_WTP_PAR_BACK;
			case IDI_WTP_SEL_OPT_1:
			case IDI_WTP_SEL_OPT_2:
			case IDI_WTP_SEL_OPT_3:
				done = true;
				break;
			case IDI_WTP_SEL_NORTH:
			case IDI_WTP_SEL_SOUTH:
			case IDI_WTP_SEL_EAST:
			case IDI_WTP_SEL_WEST:
				iDir = iSel - IDI_WTP_SEL_NORTH;
				if (hdr.roomNew[iDir] == IDI_WTP_ROOM_NONE)
					PrintMsg(IDS_WTP_CANT_GO);
				else {
					room = hdr.roomNew[iDir];
					return IDI_WTP_PAR_GOTO;
				}
				break;
			case IDI_WTP_SEL_TAKE:
				Winnie_TakeObj(room);
				Winnie_SetTakeDrop();
				break;
			case IDI_WTP_SEL_DROP:
				Winnie_DropObj(room);
				Winnie_SetTakeDrop();
				break;
			}
		}

		// jump to the script block of the selected option
		pc = hdr.opt[index].ofsOpt[iSel] - IDI_WTP_OFS_ROOM;
		opcode = *(buffer + pc);
		if (!opcode) pc++;

		// process script
		do {
			opcode = *(buffer + pc++);
			switch(opcode) {
			case IDO_WTP_GOTO_ROOM:
				opcode = *(buffer + pc++);
				iNewRoom = opcode;
				break;
			case IDO_WTP_PRINT_MSG:
				opcode = *(buffer + pc++);
				Winnie_PrintRoomStr(room, opcode);
				WaitAnyKey();
				break;
			case IDO_WTP_PRINT_STR:
				opcode = *(buffer + pc++);
				Winnie_PrintRoomStr(room, opcode);
				break;
			case IDO_WTP_DROP_OBJ:
				opcode = *(buffer + pc++);
				opcode = -1;
				Winnie_DropObjRnd();
				break;
			case IDO_WTP_FLAG_CLEAR:
				opcode = *(buffer + pc++);
				Winnie_ClearFlag(opcode);
				break;
			case IDO_WTP_FLAG_SET:
				opcode = *(buffer + pc++);
				Winnie_SetFlag(opcode);
				break;
			case IDO_WTP_GAME_OVER:
				Winnie_GameOver();
				break;
			case IDO_WTP_WALK_MIST:
				mist--;
				if (!mist) {
					room = Rnd(IDI_WTP_MAX_ROOM_TELEPORT) + 1;
					return IDI_WTP_PAR_GOTO;
				}
				break;
			case IDO_WTP_PLAY_SOUND:
				opcode = *(buffer + pc++);
				Winnie_PlaySound((ENUM_WTP_SOUND)opcode);
				break;
			case IDO_WTP_SAVE_GAME:
				Winnie_SaveGame();
				room = IDI_WTP_ROOM_HOME;
				return IDI_WTP_PAR_GOTO;
			case IDO_WTP_LOAD_GAME:
				Winnie_LoadGame();
				room = IDI_WTP_ROOM_HOME;
				return IDI_WTP_PAR_GOTO;
			case IDO_WTP_OWL_HELP:
				opcode = *(buffer + pc++);
				Winnie_ShowOwlHelp();
				break;
			case IDO_WTP_GOTO_RND:
				room = Rnd(IDI_WTP_MAX_ROOM_TELEPORT) + 1;
				return IDI_WTP_PAR_GOTO;
			default:
				opcode = 0;
				break;
			}
		} while (opcode);

		if (iNewRoom) {
			room = iNewRoom;
			return IDI_WTP_PAR_GOTO;
		}

		if (iBlock == 1)
			return IDI_WTP_PAR_OK;
	}
}

static void Winnie_Intro_DrawLogo()
{
	Winnie_DrawPic(IDS_WTP_FILE_LOGO);
	PrintStr(IDS_WTP_INTRO_0);
	Delay(64);
}

static void Winnie_Intro_DrawTitle()
{
	Winnie_DrawPic(IDS_WTP_FILE_TITLE);
	PrintStr(IDS_WTP_INTRO_1);
	Delay(64);
	WaitAnyKey();
}

static void Winnie_Intro_PlayTheme() {
	if (!Winnie_PlaySound(IDI_WTP_SND_POOH_0)) return;
	if (!Winnie_PlaySound(IDI_WTP_SND_POOH_1)) return;
	if (!Winnie_PlaySound(IDI_WTP_SND_POOH_2)) return;
}

static void Winnie_Intro()
{
	Winnie_Intro_DrawLogo();
	Winnie_Intro_DrawTitle();
	Winnie_Intro_PlayTheme();
}

static void Winnie_InitVars()
{
	memset(&game, 0, sizeof(game));
	game.fSound = 1;
	game.nObjMiss = IDI_WTP_MAX_OBJ_MISSING;
	game.nObjRet = 0;
	game.fGame[0] = 1;
	game.fGame[1] = 1;
	room = IDI_WTP_ROOM_HOME;

#ifdef _DEBUG
//Debug_Winnie_DrawRooms();
//	room = 52;
	//game.nObjMiss = 0;
#endif

}

static void Winnie_Randomize()
{
	int iObj;
	int iRoom;
	bool done;

	for (int i = 0; i < IDI_WTP_MAX_OBJ_MISSING; i++) {
		done = false;
		while (!done) {
			iObj = Rnd(IDI_WTP_MAX_OBJ - 1) + 2;
			done = true;
			for (int j = 0; j < IDI_WTP_MAX_OBJ_MISSING; j++) {
				if (game.iUsedObj[j] == iObj) {
					done = false;
					break;
				}
			}
		}

		game.iUsedObj[i] = iObj;
		
		done = false;
		while (!done) {
			iRoom = Rnd(IDI_WTP_MAX_ROOM_NORMAL) + 1;
			done = true;
			for (int j = 0; j < IDI_WTP_MAX_ROOM_OBJ; j++) {
				if (game.iObjRoom[j] == iRoom) {
					done = false;
					break;
				}
			}
		}

		game.iObjRoom[iObj] = iRoom;
	}

#ifdef _DEBUG
//	game.iUsedObj[0] = 18;
//	game.iObjRoom[18] = 28;
#endif

}

static void Winnie_GameLoop()
{
	WTP_ROOM_HDR hdr;
	Uint8 *roomdata = new Uint8[4096];
	int iBlock;

phase0:
	if (!game.nObjMiss && (room == IDI_WTP_ROOM_PICNIC)) room = IDI_WTP_ROOM_PARTY;
	Winnie_ReadRoom(room, roomdata, 4096);
	memcpy(&hdr, roomdata, sizeof(WTP_ROOM_HDR));
	Winnie_DrawRoomPic();
phase1:
	if (Winnie_GetObjInRoom(room)) {
		Winnie_PrintObjStr(Winnie_GetObjInRoom(room), IDI_WTP_OBJ_DESC);
		WaitAnyKey();
	}
phase2:
	for (iBlock = 0; iBlock < IDI_WTP_MAX_BLOCK; iBlock++) {
		if (Winnie_Parser(hdr.ofsDesc[iBlock] - IDI_WTP_OFS_ROOM, iBlock, roomdata) == IDI_WTP_PAR_BACK) {
			goto phase1;
		}
	}
	for (;;) {
		for (iBlock = 0; iBlock < IDI_WTP_MAX_BLOCK; iBlock++) {
			switch(Winnie_Parser(hdr.ofsBlock[iBlock] - IDI_WTP_OFS_ROOM, iBlock, roomdata)) {
			case IDI_WTP_PAR_GOTO:
				goto phase0;
				break;
			case IDI_WTP_PAR_BACK:
				goto phase2;
				break;
			}
		}
	}

	delete [] roomdata;
}

// Debug

static void Debug_Winnie_DrawRooms()
{
	SDL_Event event;
	char szTitle[14] = {0};
	int iRoom = 1;
	int iObj = 0;

	for (;;) {
		while (SDL_PollEvent(&event)) {
			//Winnie_DrawRoomPic(iRoom, iObj);
			ClearTextArea();
			sprintf(szTitle, "Room %d", iRoom);
			DrawStrAttrMiddle(21, IDA_DEFAULT, szTitle);
			sprintf(szTitle, "Object %d", iObj);
			DrawStrAttrMiddle(22, IDA_DEFAULT, szTitle);
			UpdateScreen();

			switch(event.type) {
			case SDL_QUIT:
				exit(0);
			case SDL_MOUSEBUTTONUP:
				switch(event.button.button) {
				case SDL_BUTTON_LEFT:
					iRoom++;
					if (iRoom > IDI_WTP_MAX_ROOM) {
						iRoom = 1;
					}
					break;
				case SDL_BUTTON_RIGHT:
					iRoom--;
					if (iRoom == 0) {
						iRoom = IDI_WTP_MAX_ROOM;
					}
					break;
				}
			case SDL_KEYDOWN:
				switch (event.key.keysym.sym) {
				case SDLK_LEFT:
					iRoom--;
					if (iRoom == 0) {
						iRoom = IDI_WTP_MAX_ROOM;
					}
					break;
				case SDLK_RIGHT:
					iRoom++;
					if (iRoom > IDI_WTP_MAX_ROOM) {
						iRoom = 1;
					}
					break;
				case SDLK_UP:
					iObj++;
					if (iObj > IDI_WTP_MAX_OBJ) {
						iObj = 0;
					}
					break;
				case SDLK_DOWN:
					iObj--;
					if (iObj < 0) {
						iObj = IDI_WTP_MAX_OBJ;
					}
					break;
				}
			}
		}
	}
}

// Init

static void Winnie_InitEngine()
{
	SetScreenPar(320, 200, (char*)ibm_fontdata);
	SetMenuPars(21, 21, IDS_WTP_SELECTION);
}

void Winnie_Init()
{
	Winnie_InitEngine();
	Winnie_InitVars();
	Winnie_Randomize();
	Winnie_Intro();
	Winnie_GameLoop();
}

// Driver

GAME (winnie, 1985, "Walt Disney Productions", "Winnie the Pooh in the Hundred Acre Wood", GAME_NO_SOUND, Winnie_Init)
