#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <time.h>
#include <SDL.h>

#include "windows.h"
#include "resource.h"

int GetFontResourceID(ENUM_FONT iFont)
{
	switch(iFont) {
	case IDI_FONT_IBM:
		return IDB_FONT_IBM;
	case IDI_FONT_MSA:
		return IDB_FONT_MICKEY;
	case IDI_FONT_WTP:
		return IDB_FONT_WINNIE;
	case IDI_FONT_APPLE2:
		return IDB_FONT_APPLE2;
	default:
		return IDB_FONT_IBM;
	}
}

SDL_Surface *LoadBitmapFromResource(int iRes)
{
	HINSTANCE		hInstance = (HINSTANCE)GetModuleHandle(NULL);
	HRSRC			hrsrc = FindResource(hInstance, MAKEINTRESOURCE(iRes), RT_BITMAP);
	HGLOBAL			hglob = LoadResource(hInstance, hrsrc);
	LPVOID			pbm = LockResource(hglob);
	LPBITMAPINFO	bmpinfo = (LPBITMAPINFO)pbm;

	// copy bitmap data to buffer

	Uint8 *buffer = new Uint8[bmpinfo->bmiHeader.biSizeImage];
	memcpy(buffer, (Uint8 *)pbm + bmpinfo->bmiHeader.biSize + 1024,		// 1024 ??
		bmpinfo->bmiHeader.biSizeImage);

	// free resource

	UnlockResource(hglob);
	FreeResource(hglob);

	// create temporary SDL_surface from buffer

	Uint32 rmask, gmask, bmask, amask = 0;

	rmask = gmask = bmask = 0xff000000;	// bitmap resources must be 256 color

	SDL_Surface *temp = SDL_CreateRGBSurfaceFrom(
		buffer, 
		bmpinfo->bmiHeader.biWidth,
		bmpinfo->bmiHeader.biHeight,
		bmpinfo->bmiHeader.biBitCount, 
		bmpinfo->bmiHeader.biWidth * bmpinfo->bmiHeader.biBitCount / 8,
		rmask, gmask, bmask, amask
	);

	// invert temp surface vertically and blit to bitmap surface

	SDL_Rect src, dst;

	SDL_Surface *bitmap = SDL_CreateRGBSurface(
		SDL_SWSURFACE,
		bmpinfo->bmiHeader.biWidth,
		bmpinfo->bmiHeader.biHeight,
		bmpinfo->bmiHeader.biBitCount,
		rmask, gmask, bmask, amask
	);

	src.x = 0;
	src.w = temp->w;
	src.h = 1;

	dst.x = 0;
	dst.w = bitmap->w;
	dst.h = 1;

	for (int y = 0; y < temp->h; y++) {
		src.y = y;
		dst.y = bitmap->h - 1 - y;
		SDL_BlitSurface(temp, &src, bitmap, &dst);
	}

	// free memory

	SDL_FreeSurface(temp);
	delete [] buffer;

	return bitmap;
}


