/*
 *  Copyright (C) 2003-2004 Curt Coder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

// screen struct

struct SYSTEM {
	SDL_Surface *screen;
	Uint8	font[2048];
	int		attr;
	bool	fCGA;
	bool	fDblSize;
	int		maxCol;
	int		maxRow;
	int		w;
	int		h;
	bool	fSound;
	bool	fDebug;
};

// default attributes

#define IDA_DEFAULT		0x0F
#define IDA_DEFAULT_REV	0xF0

// colors

#define IDC_CGA_BLACK	0x000000
#define IDC_CGA_CYAN	0x00FFFF
#define IDC_CGA_PURPLE	0xFF00FF
#define IDC_CGA_WHITE	0xFFFFFF

#define IDC_EGA_BLACK	IDC_CGA_BLACK
#define IDC_EGA_WHITE	IDC_CGA_WHITE

// color macros

#define BGCOLOR(attr) (attr & 0xf0) / 0x10
#define FGCOLOR(attr) attr & 0x0f

#define EGACOLOR(color) colorEGA[color]

#define CGACOLOREVEN(color) colorCGA[color][0]
#define CGACOLORODD(color) colorCGA[color][1]

#define BCGCOLOREVEN(color) colorBCG[color][0]
#define BCGCOLORODD(color) colorBCG[color][1]

// palettes

const Uint32 colorCGA[16][2] = {
	IDC_CGA_BLACK,	IDC_CGA_BLACK,	// 0
	IDC_CGA_BLACK,	IDC_CGA_PURPLE,	// 1
	IDC_CGA_BLACK,	IDC_CGA_CYAN,	// 2
	IDC_CGA_CYAN,	IDC_CGA_BLACK,	
	IDC_CGA_BLACK,	IDC_CGA_WHITE,	
	IDC_CGA_CYAN,	IDC_CGA_PURPLE,	// 5
	IDC_CGA_PURPLE,	IDC_CGA_BLACK,	// 6
	IDC_CGA_CYAN,	IDC_CGA_CYAN,	// 7
	IDC_CGA_PURPLE,	IDC_CGA_PURPLE,	// 8
	IDC_CGA_CYAN,	IDC_CGA_WHITE,	// 9
	IDC_CGA_PURPLE,	IDC_CGA_CYAN,	// 10
	IDC_CGA_PURPLE,	IDC_CGA_WHITE,	// 11
	IDC_CGA_WHITE,	IDC_CGA_PURPLE,	// 12
	IDC_CGA_WHITE,	IDC_CGA_BLACK,	// 13
	IDC_CGA_WHITE,	IDC_CGA_CYAN,	// 14
	IDC_CGA_WHITE,	IDC_CGA_WHITE	// 15
};

const Uint32 colorBCG[16][2] = {
	IDC_CGA_BLACK,	IDC_CGA_BLACK,	// 0
	0, 0,
	IDC_CGA_BLACK,	IDC_CGA_PURPLE,	// 2
	IDC_CGA_BLACK,	IDC_CGA_WHITE,	// 3
	0, 0,
	0, 0,
	0, 0,
	0, 0,
	IDC_CGA_PURPLE,	IDC_CGA_BLACK,	// 8
	0, 0,
	IDC_CGA_PURPLE,	IDC_CGA_PURPLE,	// A
	0, 0,
	IDC_CGA_WHITE,	IDC_CGA_BLACK,	// C
	0, 0,
	0, 0,
	IDC_CGA_WHITE,	IDC_CGA_WHITE	// F
};

const Uint32 colorEGA[16] = {
	IDC_EGA_BLACK,
	0x0000A0,
	0x00A000,
	0x00A0A0,
	0xA00000,
	0x8000A0,
	0xA05000,
	0xA0A0A0,
	0x505050,
	0x5050FF,
	0x00FF50,
	0x50FFFF,
	0xFF5050,
	0xFF50FF,
	0xFFFF50,
	IDC_EGA_WHITE
};

// screen parameters

const enum {
	IDI_COLORS_CGA,
	IDI_COLORS_PCJR
};

#define IDI_MAX_ROW_PIC		20
#define IDI_XOR_KEY			0x80

// Interface

bool GetDebug();
void SetDebug(bool fDebug);
void SetSound(bool fSound);
bool GetSound();
void SetCGA(bool fCGA);
bool GetCGA();
void SetDblSize(bool fDblSize);
void FlipSound();
void FlipCGA();
void FlipDblSize();

// Screen

void SetWindowTitle(char *szTitle);
void SetScreenPar(int w, int h, char* fontdata);
void UpdateScreen();
void ClearScreen(int attr);
void ClearGfxScreen(int color);
void ClearRow(int iRow);
void ClearTextArea();

// Fonts

int GetFontWidth();
int GetFontHeight();
void DrawStrAttr(int iRow, int iCol, int attr, char *szMsg);
void DrawStr(int iRow, int iCol, char *str);
void DrawStrAttrMiddle(int iRow, int attr, char *str);
void DrawStrAttrRight(int iRow, int attr, char *str);

// Low-Level Graphics Routines

void PutPixel(int x, int y, Uint32 color);
void PutPixelAGI(int x, int y, int color);
Uint32 GetPixel(int x, int y);
void LockScreen();
void UnlockScreen();

// Initialization

void InitGraphics();
