/*
 *  Copyright (C) 2003-2004 Curt Coder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <cstdlib>
#include <string>
#include <SDL.h>
#include "driver.h"
#include "debug.h"
#include "font.h"
#include "gfx.h"
#include "selgame.h"
#include "trollvm.h"
#include "ui.h"

static int iGameSel = NO_GAME;
static int nGames = 0;

// fill out game driver array

#define DRIVER(NAME) extern const GameDriver driver_##NAME;

DRIVER(dragon)
DRIVER(gelfling)
DRIVER(mickey)
DRIVER(winnie)
DRIVER(troll)

#undef DRIVER
#define DRIVER(NAME) &driver_##NAME,

extern const GameDriver *drivers[] = {
	//DRIVER(dragon)
	//DRIVER(gelfling)
	DRIVER(mickey)
	DRIVER(winnie)
	//DRIVER(troll)
	NULL
};

static void DrawMenu(int iSel)
{
	char szTitle[41] = {0};

	sprintf(szTitle, IDS_TITLE, IDS_VERSION);

	DrawStrAttrMiddle(2, 0x0e, szTitle);
	DrawStrAttrMiddle(4, 0x02, IDS_EMAIL);
	DrawStrAttrMiddle(5, 0x02, IDS_URL);
	DrawStrAttrMiddle(7, 0x0c, IDS_CHOOSE);

	for (int iGame = 0; iGame < nGames; iGame++)
	{
		int attr = ((iGame == iSel) ? IDA_DEFAULT_REV : IDA_DEFAULT);
		DrawStrAttrMiddle(IDI_ROW_MENU + iGame, attr, (char *)drivers[iGame]->fullname);
	}
	
	ClearTextArea();

	if (iSel != NO_GAME)
	{
		DrawStrAttrMiddle(21, IDA_DEFAULT, (char *)drivers[iSel]->company);
		DrawStrAttrMiddle(22, IDA_DEFAULT, (char *)drivers[iSel]->year);
	}

	UpdateScreen();
}

static int GetMenuSel()
{
	SDL_Event event;
	int iGameSel = NO_GAME;

	DrawMenu(iGameSel);

	for (;;)
	{
		while (SDL_PollEvent(&event))
		{
			switch (event.type)
			{
			case SDL_QUIT:
				exit(0);
			case SDL_MOUSEMOTION:
				iGameSel = (event.motion.y / GetFontHeight()) - IDI_ROW_MENU;
				if ((iGameSel < 0) || (iGameSel >= nGames)) iGameSel = NO_GAME;
				DrawMenu(iGameSel);
				break;
			case SDL_MOUSEBUTTONUP:
				if ((event.button.button == SDL_BUTTON_LEFT) && (iGameSel != NO_GAME)) return iGameSel;
				break;
			case SDL_KEYDOWN:
				if (event.key.keysym.sym == SDLK_ESCAPE) return NO_GAME;
				break;
			}
		}
	}
}

char *GetGamePath()
{
	return (char *)drivers[iGameSel]->name;
}

int main(int argc, char *argv[])
{
	int iGame;

	Init();

	while (drivers[nGames])
	{
		nGames++;
	}
	
	// process command line arguments
	for (int i = 1; i < argc; i++)
	{
		if (iGameSel == -1)
		{
			for (iGame = 0; iGame < nGames; iGame++)
			{
				if (!strcmp(argv[i], drivers[iGame]->name))
				{
					iGameSel = iGame;
					break;
				}
			}
		}

		if (!strcmp(argv[i], IDS_CMDLINE_CGA))		SetCGA		(true);
		if (!strcmp(argv[i], IDS_CMDLINE_SINGLE))	SetDblSize	(false);
		if (!strcmp(argv[i], IDS_CMDLINE_NOSOUND))	SetSound	(false);
		if (!strcmp(argv[i], IDS_CMDLINE_DEBUG))	SetDebug	(true);
	}

	// if game NOT specified on command line, show menu
	if (iGameSel == NO_GAME)
	{
		SetWindowTitle(IDS_TROLLVM);
		SetScreenPar(320, 200, (char*)ibm_fontdata);
		iGameSel = GetMenuSel();
	}

	// start selected game
	if (iGameSel != NO_GAME)
	{
		SetWindowTitle((char *)drivers[iGameSel]->fullname);
		drivers[iGameSel]->driver_init();
	}

	exit(0);
}
