/*
 *  Copyright (C) 2003-2004 Curt Coder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <cmath>
#include "sound.h"

static void FillSquareWave(double freq, char *buffer, int buflen)
{
	double coef = 2 * PI / IDI_SAMPLE_FREQUENCY * freq;

	for (int sample = 0; sample < buflen; sample++)
	{
		buffer[sample] = IDI_SAMPLE_AMPLITUDE / 2;
		if (cos(sample * coef) < 0) buffer[sample] = -buffer[sample];
	}
}

#ifdef WIN32

#include <windows.h>
#include <mmsystem.h>

HWAVEOUT hWaveOut = NULL;

bool InitSound()
{
	WAVEFORMATEX waveFormat;

	ZeroMemory(&waveFormat, sizeof(waveFormat));

	waveFormat.wFormatTag = WAVE_FORMAT_PCM;
	waveFormat.nChannels = 1;
	waveFormat.nSamplesPerSec = IDI_SAMPLE_FREQUENCY;
	waveFormat.wBitsPerSample = IDI_SAMPLE_RESOLUTION;
	waveFormat.nBlockAlign = waveFormat.nChannels * (waveFormat.wBitsPerSample / 8);
	waveFormat.nAvgBytesPerSec = waveFormat.nSamplesPerSec * waveFormat.nBlockAlign;

	if (waveOutOpen(&hWaveOut, WAVE_MAPPER, &waveFormat, NULL, 0, CALLBACK_WINDOW) != MMSYSERR_NOERROR) return false;

	return true;
}

void UninitSound()
{
	if (hWaveOut)
	{
		waveOutReset(hWaveOut);
		waveOutClose(hWaveOut);
	}
}

void PlayNote(int channel, double freq, double len)
{
	if (hWaveOut)
	{	
		if (freq)
		{
			WAVEHDR whdr;
			int buflen = (int)(IDI_SAMPLE_FREQUENCY * (IDI_SAMPLE_RESOLUTION / 8) * len);
			char *buffer = new char[buflen];

			// fill buffer with square wave
			FillSquareWave(freq, buffer, buflen);

			// prepare wave header
			memset(&whdr, 0, sizeof(whdr));
			whdr.lpData = buffer;
			whdr.dwBufferLength = (DWORD)buflen;
			waveOutPrepareHeader(hWaveOut, &whdr, sizeof(WAVEHDR));
			whdr.dwLoops = 0;

			// play sound buffer
			if (waveOutWrite(hWaveOut, &whdr, sizeof(WAVEHDR)) == MMSYSERR_NOERROR)
			{
				while (!(whdr.dwFlags & WHDR_DONE))
				{
					Sleep(1);
				}
			}

			waveOutUnprepareHeader(hWaveOut, &whdr, sizeof(WAVEHDR));

			delete [] buffer;

		} 
		else
		{
			// pause if freq = 0
			Sleep((DWORD)(len * 1000));
		}
	}
}

#endif
