/*
 *  Copyright (C) 2003-2004 Curt Coder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <cstdlib>
#include <string>
#include <ctime>
#include <SDL.h>
#include "gfx.h"
#include "ui.h"

MENUINFO menu;

// Input

void WaitAnyKey()
{
	SDL_Event event;
	
	for (;;) {
		while (SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_QUIT:
				exit(0);
			case SDL_KEYDOWN:
			case SDL_MOUSEBUTTONUP:
				return;
			}
		}
	}
}

void WaitForKey(int keycode)
{
	SDL_Event event;
	
	for (;;) {
		while (SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_QUIT:
				exit(0);
			case SDL_KEYDOWN:
				if (keycode == -1) return;
				if (keycode == event.key.keysym.sym) return;
				break;
			case SDL_MOUSEBUTTONUP:
				if (event.button.button == SDL_BUTTON_LEFT) return;
				break;
			}
		}
	}
}

bool GetSelYN()
{
	SDL_Event event;

	for (;;) {
		while (SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_QUIT:
				exit(0);
			case SDL_MOUSEBUTTONUP:
				switch(event.button.button) {
				case SDL_BUTTON_LEFT:
					return true;
				case SDL_BUTTON_RIGHT:
					return false;
				}
				break;
			case SDL_KEYDOWN:
				switch (event.key.keysym.sym) {
				case SDLK_y:
					return true;
				case SDLK_n:
					return false;
				}
				break;
			}
		}
	}
}

// Miscellanous

void XOR80(char *buffer)
{
	for (size_t i = 0; i < strlen(buffer); i++) if (buffer[i] & 0x80) buffer[i] ^= 0x80;
}

void Delay(int ticks)
{
	SDL_Delay(ticks * 10);
}

int Rnd(int hi)
{
	return (rand() * hi / (RAND_MAX + 1));
}

// Print

void PrintStr(char *szMsg)
{
	ClearTextArea();
	DrawStr(menu.nRowMsg, 0, szMsg);
	UpdateScreen();
}

void PrintMsg(char *szMsg)
{
	PrintStr(szMsg);
	WaitAnyKey();
}

void PrintStrXOR(char *szMsg)
{
	XOR80(szMsg);
	PrintStr(szMsg);
}

void PrintMsgXOR(char *szMsg)
{
	XOR80(szMsg);
	PrintMsg(szMsg);
}

// Menu

void DrawMenu(char *szMenu, int iSel)
{
	ClearTextArea();
	DrawStr(menu.nRowMsg, 0, szMenu);
	DrawStr(menu.nRowMenu + iSel, 0, menu.szSel);
	UpdateScreen();
}

void GetMouseMenuSel(int *iSel, int nSel, int x, int y)
{
	if (y < menu.nRowMenu) return;
	if (nSel > y - menu.nRowMenu) *iSel = y - menu.nRowMenu;
}

// Interface

void SetMenuPars(int nRowMsg, int nRowMenu, char *szSel)
{
	menu.nRowMsg = nRowMsg;
	menu.nRowMenu = nRowMenu;
	strcpy(menu.szSel, szSel);
}
